/* File: tcl.cc
 *
 *     This file is part of minkowsky
 *
 *     Copyright (C) 2001-2002 by Rdiger Goetz
 *     Author: Rdiger Goetz <minkowsky@r-goetz.de>
 *
 *     Time-stamp: <08-Jul-2003 19:00:53 goetz>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../client.h"

void startTCL (int argc, char *argv[] )
{
 char *cp[16],cmd[256];
 int count=2,i;

  sprintf(cmd,"%s/main.tcl",tclBinPath);
  argv[1]=cmd;
  if(autoInstallMRR)
   {
    argv[argc] = "-mrr";
    argc++;
   }
  if(enableSync)
   {
    argv[argc] = "-enableSync";
    argc++;
   }
  Tk_Main(argc,argv,initApsh);
}


int initTclFunctions (Tcl_Interp *interp)
{
 Tcl_CreateCommand(interp,"send2Server",send2Server,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"getAnswer" ,getAnswer   ,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"login"    ,logIntoServer,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"weekDayByDate",weekDayByDate,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"weekDayByDateStr",weekDayByDateStr,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"currentTime",currentTime,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"currentDateTime",currentDateTime,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"timeBetween",timeBetween,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"closeApp",closeApp,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"setTagList",setTagList ,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"getTag",getTag ,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"getBinPath",getBinPath ,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"getTclBinPath",getTclBinPath ,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"checkServer",checkServer ,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"closeConnect",closeConnect,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"mouseStatus",mouseStatus,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 Tcl_CreateCommand(interp,"matchString",matchString,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 
 return true;
}

int logIntoServer(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 char cmd[256],*buffer;
 char buf[1024];
 static int lc=0;
 int reconnect=false;

 strncpy(curUserName,argv[1],31);
 curUserName[31]='\0';
 strncpy(curPasswd,argv[2],31);
 curPasswd[31]='\0';
 if(argc>3)
  if(strcmp(argv[3],"reconnect")== 0)
   reconnect=true;

 interp->result = logIntoServer(curUserName,curPasswd,reconnect);
 return TCL_OK;
}

int closeConnect(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 if(mySocket>=0)
  {
   send2DamagedSocket(mySocket,"exit");
   closeSocket();
  }
 interp->result= "done";
 return TCL_OK;
}

int send2Server(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 static char *rbuffer = NULL;
 char *buffer,*t;
 int i;

 if (rbuffer == NULL)
  rbuffer = new char [4095];

 buffer	= globalBuffer;
 strcpy(buffer,"");
 for(i=1;i<argc;i++)
  {
   if(strlen(argv[i])==0)
    strcat(buffer,"{ } ");
   else
    {
     t=strchr(argv[i],' ');
     if(t!=NULL)
      strcat(buffer,"{");
     strcat(buffer,argv[i]);
     if(t!=NULL)
      strcat(buffer,"}");
     strcat(buffer," ");
    }
  }
 strcat(buffer," ");
 send2Server(buffer,rbuffer,4095);
 interp->result=rbuffer;

 return TCL_OK;
}

int getAnswer(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 char *buffer;

 if(!getAnswer(&buffer,argv[1]) )
  interp->result = "pending";
 else
  interp->result = buffer;
 return TCL_OK;
}


int checkServer(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 strcpy(interp->result,checkServer());
 return TCL_OK;
}

int closeApp(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 closeApp(0);
 return TCL_OK;
}


int getTag(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 int x,y,i,id;

 id = (atoi(argv[1]) % 8);
 x=atoi(argv[2]);
 y=atoi(argv[3]);

 for(i=0;i<canvasListLength[id]; i++)
  {
   if(canvasList[id][i].x0<=x)
    if(canvasList[id][i].y0<=y)
     if(x<=canvasList[id][i].x1)
      if(y<=canvasList[id][i].y1)
       {
	interp->result=canvasList[id][i].tag;
	return TCL_OK;
       }
  }
 interp->result="-";
 return TCL_OK;
}

int getBinPath(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 strcpy(interp->result,binPath);
 return TCL_OK;
}
int getTclBinPath(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 strcpy(interp->result,tclBinPath);
 return TCL_OK;
}
int setTagList(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 // usages id anz list
 // list-elem = {tag x0 y0 x1 y1}
 int anz,i,id;
 char *t,*s;
 
 id = (atoi(argv[1]) % 8);

 anz = atoi(argv[2]);
 if ( canvasList[id] !=NULL)
  delete canvasList[id];
 canvasList[id] = new canvasTags [anz];
 t= (char *) argv[3];

 for(i=0;i<anz ; i++)
  {
   s=t;
   t=nextElement(t);
   strcpy(canvasList[id][i].tag,s);

   s=t;
   t=nextElement(t);
   canvasList[id][i].x0 = atoi(s);
   s=t;
   t=nextElement(t);
   canvasList[id][i].y0 = atoi(s);
   s=t;
   t=nextElement(t);
   canvasList[id][i].x1 = atoi(s);
   s=t;
   t=nextElement(t);
   canvasList[id][i].y1 = atoi(s);
  }

 canvasListLength[id]=anz;
 return(TCL_OK);
}

int matchString(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 int len;

 if(argc <3)
  interp->result = "0";
 else 
  {
   len = strlen(argv[1]);
   if(strncasecmp(argv[2],argv[1],len)==0)
    interp->result = "1";
   else
    interp->result = "0";
  }

 return TCL_OK;
}
