/* File: tcl-unix.cc
 *
 *     This file is part of minkowsky
 *
 *     Copyright (C) 2001-2002 by Rdiger Goetz
 *     Author: Rdiger Goetz <minkowsky@r-goetz.de>
 *
 *     Time-stamp: <07-Jul-2003 20:17:59 goetz>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../client.h"

// Functions in this module used outside the module:
//
// int weekDayByDate(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
// int weekDayByDateStr(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
// int currentTime(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
// int currentDateTime(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
// int timeBetween(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
// int mouseStatu

int weekDayByDate(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 struct tm tm,*tmp;
 int tim;

 tm.tm_mday = atoi(argv[1]);
 tm.tm_mon  = atoi(argv[2])-1;
 tm.tm_year = atoi(argv[3])-1900;
 tm.tm_hour = 12;
 tm.tm_min  =tm.tm_sec =0;
 tmp=&tm;
 tim = mktime(&tm);
 tmp =localtime((const time_t *)&tim);
 sprintf(interp->result,"%d",(tmp->tm_wday+6) %7);
 return(TCL_OK);
}
int weekDayByDateStr(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 struct tm tm,*tmp;
 int tim;
 int day,mon,year,dat;
 
 dat = atoi(argv[1]);
 day = dat % 100;
 mon = ((dat -day)/100) %100;
 year= (dat-day - 100 *mon) /10000;

 tm.tm_mday = day;
 tm.tm_mon  = mon;
 tm.tm_year = year;
 tm.tm_hour = 12;
 tm.tm_min  =tm.tm_sec =0;
 tmp=&tm;
 tim = mktime(&tm);
 tmp =localtime((const time_t *)&tim);
 sprintf(interp->result,"%d",(tmp->tm_wday+6) %7);
 return(TCL_OK);
}
int currentTime(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 sprintf(interp->result,"%d",(int)time(NULL));
 return TCL_OK;
}
int currentDateTime(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 time_t tim=time(NULL);
 strftime(tclMsg,255,"%Y%m%d %H %M",localtime(&tim));   
 interp->result = tclMsg;
 return TCL_OK;
}
int timeBetween(ClientData cData, Tcl_Interp *interp, int argc,const  char *argv[])
{
 int startD,startT,stopD,stopT;
 int start,stop,delta;
 struct tm tm;

 startD = atoi(argv[1]);
 startT = atoi(argv[2]);
 stopD  = atoi(argv[3]);
 stopT  = atoi(argv[4]);
 
 tm.tm_mday = startD % 100;
 tm.tm_mon  = ((startD-tm.tm_mday)/100)%100;
 tm.tm_year = (startD-100*tm.tm_mon-tm.tm_mday)/10000 -1900;
 tm.tm_min  = startT %100;
 tm.tm_hour = (startT-tm.tm_min)/100; 
 tm.tm_sec  = 0;
 tm.tm_isdst= -1;
 start      = mktime(&tm);

 if(startD>0)
  {
   tm.tm_mday = stopD % 100;
   tm.tm_mon  = ((stopD-tm.tm_mday)/100)%100;
   tm.tm_year = (stopD-100*tm.tm_mon-tm.tm_mday)/10000 -1900;
   tm.tm_min  = stopT %100;
   tm.tm_hour = (stopT-tm.tm_min)/100;
   tm.tm_sec  = 0;
   tm.tm_isdst= -1;
   stop      = mktime(&tm);
  }
 else
  stop =time(NULL);

 delta = stop-start;
 sprintf(interp->result,"%d",delta);
 return TCL_OK;
}
