#!/usr/bin/X11/tixwish
#  File: projTimeView.tcl
#
#      This file is part of minkowsky
#
#      Copyright (C) 2001-2002 by Rdiger Goetz
#      Author: Rdiger Goetz <minkowsky@r-goetz.de>
#
#      Time-stamp: <08-Jul-2003 18:58:11 goetz>
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

proc initProjTimeView { } {
    global projColorRingFG projColorRingBG projColorRingPtr projColorRingSize
    global ptvDayWidth ptvWidthByZoom
    global depSourceTask depSourceType 

    set projColorRingFG(0) "#FF0000"
    set projColorRingFG(1) "#00FF00"
    set projColorRingFG(2) "#0000FF"
    set projColorRingFG(3) "#FFFF00"
    set projColorRingFG(4) "#FF00FF"
    set projColorRingFG(5) "#00FFFF"

    set projColorRingBG(-1) "#F8F8F8"
    set projColorRingBG(0) "#FFC0C0"
    set projColorRingBG(1) "#C0FFC0"
    set projColorRingBG(2) "#C0C0FF"
    set projColorRingBG(3) "#FFFFA0"
    set projColorRingBG(4) "#FFA0FF"
    set projColorRingBG(5) "#A0FFFF"

    set projColorRingSize 6
    set projColorRingPtr 0

    set depSourceTask ""
    set depSourceType ""

    set ptvDayWidth   24
    
    global tday tmon  tyear curOffset monlen
    global ptvStartD ptvStartM ptvStartY
    global ptvStopD  ptvStopM  ptvStopY

    set startList [calcDate $tday $tmon $tyear [expr -7 - $curOffset ] ]
    set start0    [format "%04d%02d%02d" [lindex $startList 2] [lindex $startList 1] [lindex $startList 0] ]
    set start1    [format "%04d%02d%02d" [lindex $startList 2] [lindex $startList 1] 1]
    set ptvStartY [lindex $startList 2]
    set ptvStartM [lindex $startList 1]
    if { $start0 < $start1} {
	set ptvStartD [lindex $startList 0]
    } else {
	set ptvStartD 1
    } 
    set stopList     [calcDate $tday $tmon $tyear 45 ]
    set ptvStopY     [lindex $stopList 2 ]     
    set ptvStopM     [lindex $stopList 1 ]     
    set ptvStopD     [lindex $monlen $ptvStopM  ]
    if { [expr $ptvStopM % 4] == 0 } {
	if { $ptvStopM == 2 } { incr ptvStopD }
    }

    set ptvWidthByZoom(-10) 4
    set ptvWidthByZoom(-9)  6
    set ptvWidthByZoom(-8)  8
    set ptvWidthByZoom(-7)  10
    set ptvWidthByZoom(-6)  12
    set ptvWidthByZoom(-5)  14
    set ptvWidthByZoom(-4)  16
    set ptvWidthByZoom(-3)  18
    set ptvWidthByZoom(-2)  20
    set ptvWidthByZoom(-1)  22
    set ptvWidthByZoom(0)   24
    set ptvWidthByZoom(1)   30
    set ptvWidthByZoom(2)   36
    set ptvWidthByZoom(3)   42
    set ptvWidthByZoom(4)   48
    set ptvWidthByZoom(5)   56
    set ptvWidthByZoom(6)   64
    set ptvWidthByZoom(7)   72
    set ptvWidthByZoom(8)   80
    set ptvWidthByZoom(9)   100
    set ptvWidthByZoom(10)  120
}

proc resizePtvWin { } {
    global noteBookFrameWidth noteBookFrameHeight
    global ptvCanvasWidth ptvCanvasHeight
    global ptvWidth ptvHeight ptvButtonHeight
    global ptvBaseWin

#    puts "ptv button bar height: [ winfo height $ptvBaseWin.t] [ winfo reqheight $ptvBaseWin.t] [$ptvBaseWin.t cget -height]"
    
    set ptvButtonHeight  [ winfo reqheight $ptvBaseWin.t]
    set ptvWidth  $noteBookFrameWidth
    set ptvHeight $noteBookFrameHeight
    set ptvCanvasWidth [expr $ptvWidth-2]
    set ptvCanvasHeight [expr $ptvHeight - $ptvButtonHeight -2]

    $ptvBaseWin.c.f.c configure -width $ptvCanvasWidth -height $ptvCanvasHeight

}

proc drawProjTimeView { win prj silent} {
    global ptvWin ptvZoomText _allgText _dateOrder bfnt selDateCalIcon
    global ptvStartD ptvStartM ptvStartY
    global ptvStopD  ptvStopM  ptvStopY
    global ptvoStartDobj ptvStartMobj ptvStartYobj ptvStartCobj
    global ptvoStopDobj  ptvStopMobj  ptvStopYobj  ptvStopCobj
    global ptvBaseWin
    global ptvWidth ptvHeight ptvButtonHeight
    global _projText

    set ptvBaseWin $win

    frame $win.t -relief raised -bd 1
    frame $win.c -relief flat 
    pack $win.t $win.c -side top -fill both

    label $win.t.l1 -text "$_projText(ptvRange) " -font $bfnt -width 20 -anchor ne
    pack $win.t.l1 -side left

    after 500
#    puts "-> $ptvStartD $ptvStartM $ptvStartY"

    
    set ptvStartDobj $win.t.d0
    set ptvStartMobj $win.t.m0
    set ptvStartYobj $win.t.y0
    set ptvStartCobj $win.t.c0
    set ptvStopDobj  $win.t.d1
    set ptvStopMobj  $win.t.m1
    set ptvStopYobj  $win.t.y1
    set ptvStopCobj  $win.t.c1

    label $win.t.s1 -text $_allgText(dateSep) -font $bfnt
    label $win.t.s2 -text $_allgText(dateSep) -font $bfnt
    tixControl $ptvStartDobj  -variable ptvStartD \
	-state normal -min 0 -max 31 -integer 1 \
	-options {  entry.width 2  label.width 0 label.justify center } \
	-selectmode immediate 	-autorepeat true
    tixControl $ptvStartMobj  -variable ptvStartM \
	-options { entry.width 2 label.width 0 label.justify center} \
  	-state normal	-min 0 -max 12 -integer 1 \
 	-selectmode immediate -autorepeat true
    tixControl $ptvStartYobj  -variable ptvStartY \
	-options { entry.width 4}  -state normal -min 1999 -max 2100 -integer 1 \
	-selectmode immediate -autorepeat true

    button $ptvStartCobj -image [image create pixmap -data $selDateCalIcon] -command {
	set x [ winfo rootx $ptvStartCobj]
	set y [expr  [ winfo rooty $ptvStartCobj] + [ winfo height $ptvStartCobj] ]
	set dlist [ displayYearCalAtPosition $ptvStartD $ptvStartM $ptvStartY $x $y $ptvBaseWin 0 ]
	set ptvStartD [lindex $dlist 0]
	set ptvStartM [lindex $dlist 1]
	set ptvStartY [lindex $dlist 2]
    }
    if { $_dateOrder == "de" } {
#	pack  $win.v.b.sl $eddo_startDd $win.v.b.s1 $eddo_startDm $win.v.b.s2 $eddo_startDy  $eddo_startCl -side left
	pack  $ptvStartDobj $win.t.s1 $ptvStartMobj $win.t.s1 $ptvStartYobj $ptvStartCobj -side left
    } else {
#	pack  $win.v.b.sl $eddo_startDm $win.v.b.s1 $eddo_startDd $win.v.b.s2 $eddo_startDy  $eddo_startCl -side left
	pack  $ptvStartMobj $win.t.s1 $ptvStartDobj $win.t.s1 $ptvStartYobj $ptvStartCobj -side left
    }
    pack forget [ $ptvStartDobj subwidget incr]
    pack forget [ $ptvStartDobj subwidget decr]
    pack forget [ $ptvStartMobj subwidget incr]
    pack forget [ $ptvStartMobj subwidget decr]
    pack forget [ $ptvStartYobj subwidget incr]
    pack forget [ $ptvStartYobj subwidget decr]

    label $win.t.l2 -text "  $_projText(ptvRange)  " -font $bfnt
    pack $win.t.l2 -side left


    label $win.t.s3 -text $_allgText(dateSep) -font $bfnt
    label $win.t.s4 -text $_allgText(dateSep) -font $bfnt
    tixControl $ptvStopDobj  -variable ptvStopD \
 	-state normal -min 0 -max 31 -integer 1  \
 	-options {  entry.width 2  label.width 0 label.justify center } -selectmode immediate \
 	-autorepeat true
    tixControl $ptvStopMobj  -variable ptvStopM \
	-options { entry.width 2 label.width 0 label.justify center}\
  	-state normal	-min 0 -max 12 -integer 1\
 	-selectmode immediate -autorepeat true
    tixControl $ptvStopYobj  -variable ptvStopY \
	-options { entry.width 4}  -state normal -min 1999 -max 2100 -integer 1 \
	-selectmode immediate -autorepeat true

    button $ptvStopCobj -image [image create pixmap -data $selDateCalIcon] -command {
	set x [ winfo rootx $ptvStopCobj]
	set y [expr  [ winfo rooty $ptvStopCobj] + [ winfo height $ptvStopCobj] ]
	set dlist [ displayYearCalAtPosition $ptvStopD $ptvStopM $ptvStopY $x $y $ptvBaseWin 0 ]
	set ptvStopD [lindex $dlist 0]
	set ptvStopM [lindex $dlist 1]
	set ptvStopY [lindex $dlist 2]
    }
    if { $_dateOrder == "de" } {
#	pack  $win.v.b.sl $eddo_startDd $win.v.b.s1 $eddo_startDm $win.v.b.s2 $eddo_startDy  $eddo_startCl -side left
	pack  $ptvStopDobj $win.t.s3 $ptvStopMobj $win.t.s4 $ptvStopYobj $ptvStopCobj -side left
    } else {
#	pack  $win.v.b.sl $eddo_startDm $win.v.b.s1 $eddo_startDd $win.v.b.s2 $eddo_startDy  $eddo_startCl -side left
	pack  $ptvStopMobj $win.t.s3 $ptvStopDobj $win.t.s4 $ptvStopYobj $ptvStopCobj -side left
    }
    pack forget [ $ptvStopDobj subwidget incr]
    pack forget [ $ptvStopDobj subwidget decr]
    pack forget [ $ptvStopMobj subwidget incr]
    pack forget [ $ptvStopMobj subwidget decr]
    pack forget [ $ptvStopYobj subwidget incr]
    pack forget [ $ptvStopYobj subwidget decr]

    $ptvStartDobj configure -validatecmd validateProjStartDatesDay   -command ptvReDrawAll
    $ptvStartMobj configure -validatecmd validateProjStartDatesMon   -command ptvReDrawAll
    $ptvStartYobj configure -validatecmd validateProjStartDatesYear  -command ptvReDrawAll
    $ptvStopDobj configure -validatecmd validateProjStopDatesDay     -command ptvReDrawAll
    $ptvStopMobj configure -validatecmd validateProjStopDatesMon     -command ptvReDrawAll
    $ptvStopYobj configure -validatecmd validateProjStopDatesYear    -command ptvReDrawAll

    
    global    ptvZoom bfnt bfnt12
    set ptvZoom 0
    tixOptionMenu $win.t.z  -options {
	menubutton.width 3 \
	    menu.width 3 \
	    label.width 6}\
	-label $_projText(ptvZoom)
    for {set z -10 } {$z <= 10} { incr z } {
	$win.t.z add command $z -label [format "%5d" $z]
    }
    
    $win.t.z configure -variable ptvZoom
    $win.t.z subwidget menu configure -font  $bfnt12
    $win.t.z subwidget menubutton configure -font  $bfnt12
    $win.t.z subwidget label configure -font  $bfnt
#    tixControl $win.t.z  -variable ptvZoom \
# 	-state normal -min -10 -max 10 -integer 1  \
 #	-options {  entry.width 3  label.width 0 label.justify left entry.justify right } -selectmode immediate \
 #	-autorepeat false -label "Zoom  " -command ptvUpdateZoom -allowempty 1
    label $win.t.sz -text "   " -width 4 -font $bfnt
    pack $win.t.sz $win.t.z -side right
    

#     set ptvWidth  $width
#     set ptvHeight $height
#     set ptvButtonHeight 20
#     set ptvCanvasWidth $ptvHeight
#     set ptvCanvasHeight [expr $ptvHeight - $ptvButtonHeight]

    drawProjTimeViewCanvas  $win.c  1


    ptvDraw $prj

    ptvBindActions 
    updateGrid $silent
    $win.t.z configure -command ptvUpdateZoom
}

proc ptvUpdateZoom { dummy } {
    global ptvZoom ptvWidthByZoom ptvDayWidth ptvZoomText

    if { $ptvZoom != "" } {
	set ptvDayWidth $ptvWidthByZoom($ptvZoom)
    }
 
    ptvReDrawAll 1
}

proc validateProjStartDatesDay { val } {
    global ptvStartD ptvStartM ptvStartY 
    global ptvStopD  ptvStopM  ptvStopY
    global monlen

    if { $val <1} { set val 1}
    set ml [ lindex $monlen  $ptvStartM ]
    if { $ptvStartM == 2} {
	if { [expr $ptvStartY % 4] == 0} {incr ml }
    }
    if { $val > $ml } { set val  $ml}

    if { [validateProjDates $ptvStartY $ptvStartM $val  $ptvStopY $ptvStopM $ptvStopD ] == 0 } { return $val }
    return $ptvStartD
}
proc validateProjStartDatesMon { val } {
    global ptvStartD ptvStartM ptvStartY 
    global ptvStopD  ptvStopM  ptvStopY
    global monlen

    if { $val <1} { set val 1}
    if { $val > 12  } { set val 12}

    if { [validateProjDates $ptvStartY $val $ptvStartD $ptvStopY $ptvStopM $ptvStopD ] == 0 } { return $val }
    return $ptvStartM
}
proc validateProjStartDatesYear { val } {
    global ptvStartD ptvStartM ptvStartY 
    global ptvStopD  ptvStopM  ptvStopY

    if { [validateProjDates $val $ptvStartM $ptvStartD $ptvStopY $ptvStopM $ptvStopD ] == 0 } { return $val }
    return $ptvStartY
}
proc validateProjStopDatesDay { val } {
    global ptvStartD ptvStartM ptvStartY 
    global ptvStopD  ptvStopM  ptvStopY
    global monlen

    if { $val <1} { set val 1}
    set ml [ lindex $monlen  $ptvStopM ]
    if { $ptvStopM == 2} {
	if { [expr $ptvStopY % 4] == 0} {incr ml }
    }
    if { $val > $ml } { set val  $ml}

    if { [validateProjDates $ptvStartY $ptvStartM $ptvStartD $ptvStopY $ptvStopM $val ] == 0 } { return $val }
    return $ptvStopD
}
proc validateProjStopDatesMon { val } {
    global ptvStartD ptvStartM ptvStartY 
    global ptvStopD  ptvStopM  ptvStopY
    global monlen

    if { $val <1} { set val 1}
    if { $val > 12  } { set val 12}

    if { [validateProjDates $ptvStartY $ptvStartM $ptvStartD $ptvStopY $val $ptvStopD ] == 0 } { return $val }
    return $ptvStopM
}
proc validateProjStopDatesYear { val } {
    global ptvStartD ptvStartM ptvStartY 
    global ptvStopD  ptvStopM  ptvStopY

    if { [validateProjDates $ptvStartY $ptvStartM $ptvStartD $val $ptvStopM $ptvStopD ] == 0 } { return $val }
    return $ptvStopY
}
proc validateProjDates { startDy startDm startDd  stopDy stopDm stopDd } { 
    global ptvStartD ptvStartM ptvStartY 
    global ptvStopD  ptvStopM  ptvStopY

    set start [format "%04d02d%02d" $startDy $startDm $startDd ]
    set stop  [format "%04d02d%02d" $stopDy  $stopDm  $stopDd  ]
    if { $stop < $start } {
	set ptvStopY $ptvStartY
	set ptvStopM $ptvStartM
	set ptvStopD $ptvStartD
	return -1
    }
    return 0
}

#######################
#
#The Canvas
#
proc drawProjTimeViewCanvas { win  withWE} {
    global ptvStartDate ptvStopDate ptvDateWeekEnds
    global ptvDayWidth ptvDateCoords ptvCanvas ptvBalloon
    global ptvSwin ptvCanvasHeight ptvCanvasWidth ptvInnerCanvasHeight
    global ptvStartD ptvStartM ptvStartY
    global ptvStopD  ptvStopM  ptvStopY



#    puts "STOP: /$ptvStopY/  /$ptvStopM/  /$ptvStopD/"
    set ptvStartDate [format "%4d%02d%02d" $ptvStartY $ptvStartM $ptvStartD ]
    set ptvStopDate  [format "%4d%02d%02d" $ptvStopY  $ptvStopM  $ptvStopD ]
    set ptvDateWeekEnds $withWE
#    set ptvCanvasWidth $cwidth
    set cwidth [expr $ptvDayWidth * [calcDateDiff $ptvStartDate $ptvStopDate]]
#    puts "SIZE $cwidth $cheight // $ptvCanvasWidth"
    
#    puts "PTV canvas size $ptvCanvasWidth x $ptvCanvasHeight"
    frame  $win.f -relief sunken -bd 1
    pack $win.f -side top -fill both -expand 1
    tixCObjView $win.f.c -height $ptvCanvasHeight -width $ptvCanvasWidth
    $win.f.c subwidget vsb configure -width 12
    $win.f.c subwidget hsb configure -width 12
    set ptvCanvas [$win.f.c subwidget canvas  ]
    $ptvCanvas configure -bg grey96 -width $cwidth
    pack $win.f.c -side top -fill both -expand 1

    set ptvInnerCanvasHeight [expr $ptvCanvasHeight -20 ]
    ptvDrawGrid $ptvCanvas [expr $ptvInnerCanvasHeight -20 ]
    set ptvSwin $win.f.c
    $ptvSwin adjustscrollregion

    set ptvBalloon $win.b
    tixBalloon $ptvBalloon  -initwait 2000 -state balloon


    bind [$ptvSwin subwidget hsb ] <Button-4> { ptvCanvasHscroll -10 units }
    bind [$ptvSwin subwidget hsb ] <Button-5> { ptvCanvasHscroll +10 units}
    bind [$ptvSwin subwidget hsb ] <Shift-Button-4> { ptvCanvasHscroll  -1 pages }
    bind [$ptvSwin subwidget hsb ] <Shift-Button-5> { ptvCanvasHscroll  +1 pages}
    bind [$ptvSwin subwidget vsb ] <Button-4> { ptvCanvasVscroll -10 units }
    bind [$ptvSwin subwidget vsb ] <Button-5> { ptvCanvasVscroll +10 units}
    bind [$ptvSwin subwidget vsb ] <Shift-Button-4> { ptvCanvasVscroll  -1 pages }
    bind [$ptvSwin subwidget vsb ] <Shift-Button-5> { ptvCanvasVscroll  +1 pages}
    bind [$ptvSwin subwidget canvas ] <Button-4> { ptvCanvasVscroll -10 units }
    bind [$ptvSwin subwidget canvas ] <Button-5> { ptvCanvasVscroll +10 units}
    bind [$ptvSwin subwidget canvas ] <Shift-Button-4> { ptvCanvasVscroll  -1 pages }
    bind [$ptvSwin subwidget canvas ] <Shift-Button-5> { ptvCanvasVscroll  +1 pages}

}
proc ptvKeyScroll {key} {
    puts $key
}
proc ptvCanvasHscroll {amt unit} {
    global ptvSwin

    set win [ $ptvSwin subwidget hsb]
    set cmd [ $win cget -command ]
    eval "$cmd scroll $amt $unit"
}
proc ptvCanvasHscrollToDate { date} {
    global ptvSwin ptvDateCoords ptvLastDateCoord

    set win [ $ptvSwin subwidget hsb]
    set cmd [ $win cget -command ]
    set f [expr double($ptvDateCoords($date))  / double($ptvLastDateCoord) ]
#    puts "$ptvDateCoords($date)  / $ptvLastDateCoord -> $f"
    eval "$cmd moveto $f"
}
proc ptvCanvasVscroll  {amt unit} {
    global ptvSwin    
    set win [ $ptvSwin subwidget vsb]
    set cmd [ $win cget -command ]
    eval "$cmd scroll $amt $unit"
}


proc ptvDrawGrid { cwin height } {
    global ptvStartDate ptvStopDate ptvDateWeekEnds ptvStartM ptvCanvasWidth ptvLastDateCoord
    global ptvDayWidth ptvDateCoords weekDaysAbk weekDays
    global fnt10 bfnt monNames fnt8 fnt6 fnt7
    global ptvHeadBottom

    array  unset  ptvDateCoords
    set date $ptvStartDate
    set dx0 16
#    if {$dx0 < 6 } { set dx0 6 }  elseif { $dx0 > 12 } { set dx0  12 }
    set xk0 $dx0
    set xk1 [ expr $xk0 +$ptvDayWidth]
    set xkm [expr int(floor(($xk0 +$xk1 ) /2.)) ]

    array unset monX0
    array unset monX1
    set ms [ expr $date - ($date %100) ]
#    set ms $ptvStartM
    set monX0($ms) $xk0


    if { $ptvDayWidth > 16 } {      set ptvHeadBottom 48 
    } elseif { $ptvDayWidth > 12 } {set ptvHeadBottom 44
    } elseif { $ptvDayWidth > 9 } { set ptvHeadBottom 43
    } elseif { $ptvDayWidth > 7 } { set ptvHeadBottom 34
    } else { set ptvHeadBottom 28 }
    $cwin create rectangle 0 0 $dx0 $ptvHeadBottom -fill white -outline white -tag head
    $cwin create rectangle 0 $ptvHeadBottom $dx0 $height -fill "#FFF0C0" -outline "#FFF0C0" -tag grid
    $cwin create line 0 24 $dx0 24 -fill black -tag head
    $cwin create line 0 $ptvHeadBottom $dx0 $ptvHeadBottom -fill black -tag head
    
    set dlist [date2list $date]
    set wd [ eval "weekDayByDate $dlist " ]
    $cwin create line $xk0 $ptvHeadBottom $xk0 $height -fill black -tag grid 
    if { [expr $date %100] == 1 } {
	$cwin create line [expr $xk0 -1 ] 1 [expr $xk0 -1] $height -fill black -tag grid 
    }

 
#    puts $ptvCanvasWidth
    while { $date <= $ptvStopDate || $xk1 < $ptvCanvasWidth} {
	set dlist [date2list $date]

	set wdstr [lindex $weekDaysAbk $wd]
	set wdcol "black"
	set wdcol2 "white"

	if {$wd >=5 } { 
	    set wdcol "red" 
	    set wdcol2 "red" 
	}
	set gcol "grey66"

	if { $wd == 6 } { set gcol "black" }

	if { $ptvDayWidth > 7 } {
	    $cwin create rectangle $xk0 24 $xk1 $ptvHeadBottom -fill white -outline black -tag head
	} else {
	    $cwin create rectangle $xk0 24 $xk1 $ptvHeadBottom -fill $wdcol2 -outline black -tag head
	}
	$cwin create line $xk1 $ptvHeadBottom $xk1 $height -fill $gcol -tag grid

	if { $ptvDayWidth > 40 } {
	    set wlstr [ lindex $weekDays $wd]
	    set wstr [strTrunc3 $wlstr [expr $ptvDayWidth -2 ] $fnt10 [string length $wlstr ] ]
	    $cwin create text $xkm 36 -text " $wstr " -font $fnt10 -anchor n -justify center -tag head -fill $wdcol
	    $cwin create text $xkm 25 -text "[lindex $dlist 0]." -font $fnt10 -anchor n -justify center -tag head -fill $wdcol	    
	} elseif { $ptvDayWidth > 29 } {
	    set wlstr [ lindex $weekDays $wd]
	    set wstr "[string range  $wlstr 0 3 ]."
	    $cwin create text $xkm 36 -text " $wstr " -font $fnt10 -anchor n -justify center -tag head -fill $wdcol
	    $cwin create text $xkm 25 -text "[lindex $dlist 0]." -font $fnt10 -anchor n -justify center -tag head -fill $wdcol	    
	} elseif { $ptvDayWidth > 16 } {
	    $cwin create text $xkm 36 -text " $wdstr " -font $fnt10 -anchor n -justify center -tag head -fill $wdcol
	    $cwin create text $xkm 25 -text "[lindex $dlist 0]." -font $fnt10 -anchor n -justify center -tag head -fill $wdcol
	} elseif { $ptvDayWidth > 12 } {
	    $cwin create text $xkm 34 -text " $wdstr " -font $fnt8 -anchor n -justify center -tag head -fill $wdcol
	    $cwin create text $xkm 25 -text "[lindex $dlist 0]." -font $fnt8 -anchor n -justify center -tag head -fill $wdcol
	} elseif { $ptvDayWidth > 9 } {
	    $cwin create text [expr $xkm -1 ] 33 -text [string range $wdstr 0 0 ] -font $fnt7 -anchor n -justify center -tag head -fill $wdcol
	    $cwin create text $xkm 24 -text "[lindex $dlist 0]" -font $fnt7 -anchor n -justify center -tag head -fill $wdcol
	} elseif { $ptvDayWidth > 7 } {
	    $cwin create text [expr $xkm -1 ] 24 -text [string range $wdstr 0 0 ] -font $fnt7 -anchor n -justify center -tag head -fill $wdcol
	}

	set ptvDateCoords($date) $xk0
#	puts "$date -> $ptvDateCoords($date) "

	set mso $ms
	set date [ calcDateStr $date 1]
	set wd [ expr ($wd +1) % 7 ]
	set ms [ expr $date - ($date %100) ]

	if { $mso != $ms } {
	    set monX1($mso) $xk1
#	    puts "ms=$mso $date"
	    $cwin create line $xk1 1 $xk1 $height -fill black -tag grid
	    incr xk1
	    incr xkm
	    $cwin create line $xk1 1 $xk1 $height -fill black -tag grid
	    if { $date <= $ptvStopDate || $xk1 < $ptvCanvasWidth} {
		set monX0($ms) $xk1
	    }
	}
	set xk0 $xk1
	incr xk1 $ptvDayWidth
	incr xkm $ptvDayWidth
    }

    set monX1($ms) $xk0
    $cwin create rectangle [expr $xk0+1] 0 [expr $xk0 + $dx0] $ptvHeadBottom -fill white -outline white -tag head
    $cwin create rectangle [expr $xk0+1] $ptvHeadBottom [expr $xk0 + $dx0] $height -fill "#FFF0C0" -outline "#FFF0C0" -tag grid
    $cwin create line $xk0  24 [expr $xk0 + $dx0] 24 -fill black -tag head
    $cwin create line $xk0 $ptvHeadBottom  [expr $xk0 + $dx0 +1] $ptvHeadBottom -fill black -tag head
    set ptvLastDateCoord [expr $xk0 + $dx0]
    
    set monCol(0) "#FDFAF4"
    set monCol(1) "#FDF4FA"

    foreach ms [array names monX0] {
	set year [expr ($ms /10000) ]
	set mon  [expr ($ms /100) % 100 ]
	set mm   [expr $mon %2 ]
	
	set txt [format "%s %4d" [lindex $monNames $mon] $year]
	set txtWidth [font measure $bfnt $txt]

	$cwin create rectangle  $monX0($ms) 0 $monX1($ms) 24 -outline black -fill white  -tag "head head-$ms"
	set xkm [ expr int (floor( ($monX0($ms) +$monX1($ms))/2.))]
	$cwin create text $xkm 22 -text $txt -anchor s -justify center -font $bfnt -tag head
	set box [$cwin bbox head-$ms]
	set boxWidth [ expr [lindex $box 2] - [lindex $box 0] ]
	if { [expr $txtWidth *4 + 8* $ptvDayWidth] < $boxWidth } {
	    $cwin create text [expr $monX0($ms)+8] 22 -text "$txt ..." -anchor sw -justify left -font $bfnt -tag head
	    $cwin create text [expr $monX1($ms)-8] 22 -text "... $txt" -anchor se -justify right -font $bfnt -tag head
	}
	$cwin create rectangle $monX0($ms) 48 $monX1($ms) $height -outline $monCol($mm) -fill $monCol($mm) -tag bg
    }


    $cwin lower grid head
    $cwin lower bg grid
}

proc updateGrid { silent } {
    global ptvCanvas ptvSwin ptvInnerCanvasHeight

    set fgList [ $ptvCanvas bbox projFG ]
    set bgList [ $ptvCanvas bbox projBG ]
    if { $fgList != "" } {
	set ym1 [lindex $fgList 3]
    } else {
	set ym1 0
    }
    if { $bgList != "" } {
	set ym2 [lindex $bgList 3]
    } else {
	set ym2 0
    }

#    set ym1 [lindex [ $ptvCanvas bbox projFG ] 3 ]
#    set ym2 [lindex [ $ptvCanvas bbox projBG ] 3 ]
#    puts $ym1
    if { $ym1 > $ym2 } { set ym $ym1 }  else {  set ym $ym2 }
    incr ym 8
    if { $ym < $ptvInnerCanvasHeight } { set  ym  $ptvInnerCanvasHeight}
#    puts "ym= $ym"

    set idxList [$ptvCanvas find withtag grid]
    set idxList [concat $idxList [$ptvCanvas find withtag bg]]
    foreach idx $idxList {
	set clist [ $ptvCanvas coords $idx]
	set clist [lreplace $clist 3 3 $ym]
	$ptvCanvas coords $idx $clist
    }

    ptvArrangeObjets
    if { $silent == 0 } {
	update
    }
    $ptvSwin adjustscrollregion
}

    

proc ptvBindActions { } {
    global ptvCanvas 
   


    $ptvCanvas bind taskDepDONE    <ButtonPress-1> { ptvRecordButtonPressObj %x %y DONE task}
    $ptvCanvas bind taskDepKILL    <ButtonPress-1> { ptvRecordButtonPressObj %x %y KILL task}
    $ptvCanvas bind taskDepHOLD    <ButtonPress-1> { ptvRecordButtonPressObj %x %y HOLD task}
    $ptvCanvas bind taskDepWIP     <ButtonPress-1> { ptvRecordButtonPressObj %x %y WIP  task}
    $ptvCanvas bind taskMileStone  <ButtonPress-1> { ptvRecordButtonPressObj %x %y MILES  task}
    $ptvCanvas bind taskComplete   <ButtonPress-1> { ptvRecordButtonPressObj %x %y COMPL  task}
    $ptvCanvas bind taskExpand     <ButtonPress-1> { ptvRecordButtonPressObj %x %y Expand task }
    $ptvCanvas bind taskModify     <ButtonPress-1> { ptvRecordButtonPressObj %x %y Modify task }
    $ptvCanvas bind taskMemo       <ButtonPress-1> { ptvRecordButtonPressObj %x %y Memo   task }
    $ptvCanvas bind taskStatus     <ButtonPress-1> { ptvRecordButtonPressObj %x %y Status  task}
    $ptvCanvas bind projDepDONE    <ButtonPress-1> { ptvRecordButtonPressObj %x %y DONE proj }
    $ptvCanvas bind projDepKILL    <ButtonPress-1> { ptvRecordButtonPressObj %x %y KILL proj}
    $ptvCanvas bind projDepHOLD    <ButtonPress-1> { ptvRecordButtonPressObj %x %y HOLD proj}
    $ptvCanvas bind projDepWIP     <ButtonPress-1> { ptvRecordButtonPressObj %x %y WIP  proj}
    $ptvCanvas bind projComplete   <ButtonPress-1> { ptvRecordButtonPressObj %x %y COMPL  proj}
    $ptvCanvas bind projExpand     <ButtonPress-1> { ptvRecordButtonPressObj %x %y Expand  proj}
    $ptvCanvas bind projModify     <ButtonPress-1> { ptvRecordButtonPressObj %x %y Modify  proj}
    $ptvCanvas bind projMemo       <ButtonPress-1> { ptvRecordButtonPressObj %x %y Memo   proj }
    $ptvCanvas bind projStatus     <ButtonPress-1> { ptvRecordButtonPressObj %x %y Status  proj}
    $ptvCanvas bind projFG         <ButtonPress-1> { ptvRecordButtonPressObj %x %y Proj proj }

     $ptvCanvas bind taskDepDONE    <ButtonRelease-1> { ptvTaskAction %x %y DONE task}
     $ptvCanvas bind taskDepKILL    <ButtonRelease-1> { ptvTaskAction %x %y KILL task}
     $ptvCanvas bind taskDepHOLD    <ButtonRelease-1> { ptvTaskAction %x %y HOLD task}
     $ptvCanvas bind taskDepWIP     <ButtonRelease-1> { ptvTaskAction %x %y WIP  task}
     $ptvCanvas bind taskMileStone  <ButtonRelease-1> { ptvTaskAction %x %y MILES  task}
     $ptvCanvas bind taskComplete   <ButtonRelease-1> { ptvTaskAction %x %y COMPL  task}
     $ptvCanvas bind taskExpand     <ButtonRelease-1> { ptvTaskAction %x %y Expand task }
     $ptvCanvas bind taskModify     <ButtonRelease-1> { ptvTaskAction %x %y Modify  task}
     $ptvCanvas bind taskMemo       <ButtonRelease-1> { ptvTaskAction %x %y Memo    task}
     $ptvCanvas bind taskStatus     <ButtonRelease-1> { ptvTaskAction %x %y Status  task}
     $ptvCanvas bind taskFG         <ButtonRelease-1> { ptvTaskAction %x %y Task  task}
     $ptvCanvas bind projDepDONE    <ButtonRelease-1> { ptvProjAction %x %y DONE proj}
     $ptvCanvas bind projDepKILL    <ButtonRelease-1> { ptvTaskAction %x %y KILL proj}
     $ptvCanvas bind projDepHOLD    <ButtonRelease-1> { ptvTaskAction %x %y HOLD proj}
     $ptvCanvas bind projDepWIP     <ButtonRelease-1> { ptvTaskAction %x %y WIP  proj}
     $ptvCanvas bind projComplete   <ButtonRelease-1> { ptvTaskAction %x %y COMPL  proj}
     $ptvCanvas bind projExpand     <ButtonRelease-1> { ptvTaskAction %x %y Expand  proj}
     $ptvCanvas bind projModify     <ButtonRelease-1> { ptvTaskAction %x %y Modify  proj}
     $ptvCanvas bind projMemo       <ButtonRelease-1> { ptvTaskAction %x %y Memo    proj}
     $ptvCanvas bind projStatus     <ButtonRelease-1> { ptvTaskAction %x %y Status  proj}
     $ptvCanvas bind projFG         <ButtonRelease-1> { ptvTaskAction %x %y Proj  proj}

    $ptvCanvas bind dep             <ButtonRelease-3> { ptvCheckRemoveDep %X %Y}
#    $ptvCanvas bind depInTask       <ButtonRelease-3> { ptvCheckRemoveDep }
}

proc ptvArrangeObjets { } {
    global ptvCanvas

    $ptvCanvas lower grid head
    $ptvCanvas lower bg grid

    if { [ $ptvCanvas find withtag taskBG] != "" } {
	$ptvCanvas raise taskBG grid
    }
    if { [ $ptvCanvas find withtag projBG] != "" } {
	$ptvCanvas raise projBG grid
    }
    if { [ $ptvCanvas find withtag dep] != "" } {
	$ptvCanvas raise dep grid
 	if { [ $ptvCanvas find withtag dep-open] != "" } {
 	    if { [ $ptvCanvas find withtag dep-done] != "" } {
 		$ptvCanvas raise dep-open dep-done
 	    }
 	}
	
	$ptvCanvas raise taskBG dep
	$ptvCanvas raise projBG dep
 	if { [ $ptvCanvas find withtag dep-open] != "" } {
 	    $ptvCanvas raise taskBG dep-open
 	    $ptvCanvas raise projBG dep-open
 	}
    } 

    if { [ $ptvCanvas find withtag taskFG] != "" } {
	if { [ $ptvCanvas find withtag taskBG] != "" } {
	    $ptvCanvas raise taskFG taskBG
	} else {
	    $ptvCanvas raise taskFG grid
	}
    }
    if { [ $ptvCanvas find withtag projFG] != "" } {
	if { [ $ptvCanvas find withtag projBG] != "" } {
	    $ptvCanvas raise projFG projBG
	} else {
	    $ptvCanvas raise projFG grid
	}
    }
    if  { [ $ptvCanvas find withtag depInTask] != "" } {
	 if { [ $ptvCanvas find withtag projFG] != "" } {
	      $ptvCanvas raise depInTask projFG
	 }
	 if { [ $ptvCanvas find withtag taskFG] != "" } {
	      $ptvCanvas raise depInTask taskFG
	 }
    }
}
proc ptvReDrawAll {dummy } {
    global ptvCanvas ptvSwin ptvInnerCanvasHeight
    global ptvStartD ptvStartM ptvStartY
    global ptvStopD  ptvStopM  ptvStopY
    global ptvStartDate ptvStopDate ptvDateWeekEnds
    global ptvTaskDepSourceX  ptvTaskDepSourceY ptvTaskDepSourceCount


    array unset ptvTaskDepSourceX
    array unset ptvTaskDepSourceY
    array unset ptvTaskDepSourceCount
    set ptvStartDate [format "%4d%02d%02d" $ptvStartY $ptvStartM $ptvStartD ]
    set ptvStopDate  [format "%4d%02d%02d" $ptvStopY  $ptvStopM  $ptvStopD ]

    $ptvCanvas delete all 
    ptvDrawGrid $ptvCanvas $ptvInnerCanvasHeight
    $ptvSwin adjustscrollregion

    ptvDraw -1
    ptvBindActions
    updateGrid 0
}

proc ptvReDraw { dummy } {
    global ptvCanvas
    global ptvCurrentProj
    
    $ptvCanvas delete dep
    $ptvCanvas delete taskFG
    $ptvCanvas delete taskBG
    $ptvCanvas delete projFG
    $ptvCanvas delete projBG
    ptvDraw $ptvCurrentProj
#     global tdd_projList
#     puts "PROJ 1: $tdd_projList(1)\n"
#     puts "PROJ 2: $tdd_projList(2)\n"
}
proc ptvDraw { projID } {
    global tdd_projList tdd_todoPID
    global projSortMode projSortMode2 projSortMode3
    global ptvCurrentProj ptvHeadBottom
    
    set ptvCurrentProj $projID
    set y [expr $ptvHeadBottom +6 ]
    #FIXME proj und Task durcheinander
    if { $projID <0 } {
	set pidList ""
	foreach pid [array names tdd_projList] {
	    if { [lindex $tdd_projList($pid) 1] == "PROJ" } {
		if { [lindex $tdd_projList($pid) 16] == 0 } {
#		    lappend pidList "-$pid"
		    lappend pidList $pid
		}
	    }
	}
	set projSortMode  "start"
	set projSortMode2 "dring"
	set projSortMode3 "stop"
    
#	puts $pidList

	set pList [lsort -command compareProj "-decreasing" $pidList]  
#	puts $pidList
#	puts $pList
	foreach pid $pList {
	    set y [ptvDrawProj $y $pid ]
	    incr y 24
	}
    } else { 
	ptvDrawProj $y $projID
    }
}

proc ptvDrawProj { y id } {
    global tdd_projList 
    global todoStatIcon dringIcon quitIcon taskExpandedIcon taskCollapsedIcon taskModifySmallIcon taskModifyIcon
    global projColorRingFG projColorRingBG projColorRingPtr projColorRingSize ptvProjColorID
    global memoIcon memoIcon16 taskModifyIcon16 taskExpandedIcon16  taskCollapsedIcon16
    global ptvDateCoords ptvCanvas
    global bfnt16 fnt bfnt12 bfnt10
    global projExpanded taskHidden
    global ptvStartDate ptvStopDate
    global _projText

    set stat    [lindex $tdd_projList($id) 4]
    set title   [lindex $tdd_projList($id) 5]
    set start   [lindex $tdd_projList($id) 6]
    set stop    [lindex $tdd_projList($id) 7]
    set dring   [lindex $tdd_projList($id) 10]
    set descr   [lindex $tdd_projList($id) 14]

    if { $start > $ptvStopDate || $stop <= $ptvStartDate } { return $y}

    if { [ array names projExpanded $id] == "" } {
	set projExpanded($id) 1
	set ptvProjColorID($id) $projColorRingPtr
	incr projColorRingPtr
	if { $projColorRingPtr > $projColorRingSize } { set projColorRingPtr 0 }
    }

    set colid $ptvProjColorID($id)
    set fgcol $projColorRingFG($colid)
    set bgcol $projColorRingBG($colid)

    global ptvStartDate potvStopDate
    set ptvProjLeftOpen 0
    set ptvProjRightOpen 0
#    puts "PROJ: $start - $stop ; display : $ptvStartDate - $ptvStopDate"
    if { $start >= $ptvStartDate } {
	set xk0 $ptvDateCoords($start)
    } else { 
	set xk0 $ptvDateCoords($ptvStartDate)
	set ptvProjLeftOpen 1
    }
    if { $stop <= $ptvStopDate } {
	set xk1 $ptvDateCoords($stop)
    } else { 
	set xk1 $ptvDateCoords($ptvStopDate)
	set ptvProjRightOpen 1
    }

    set yk0 $y
    incr xk0
    incr xk1 -1
    set ykt $yk0
    set dpx [expr $xk1-$xk0]

    set projWinSize 0
    if { $dpx  < 300 } {
	if { $dpx >160 } { set projWinSize 1 
	} elseif { $dpx >144 } { set projWinSize 2
	} elseif { $dpx >105 } { set projWinSize 3 
	} elseif { $dpx > 70 } { set projWinSize 4 
	} elseif { $dpx > 45 } { set projWinSize 5 
	} elseif { $dpx > 25 } { set projWinSize 6 
	} else { set projWinSize 7 }
    }

#    puts "PROJ $title : $xk1 $xk0 -> $dpx -> SIZE=$projWinSize"
    
    set mwidth [ expr ($xk1 -4) - ($xk0 +64) ]
    set twidth  [expr ($xk1-94) - ($xk0 +64) -4 ]
    if { $projWinSize < 3 } {
	ptvDrawIconBox $ptvCanvas [expr $xk0+2] [expr $yk0+2] 24 24 "proj-$id projStatus projFG"  $todoStatIcon($stat)
	ptvDrawIconBox $ptvCanvas [expr $xk0+32] [expr $yk0+2] 24 24 "proj-$id projDring projFG"  $dringIcon($dring) 
	if { $projExpanded($id) == 0 } {
	    ptvDrawIconBox $ptvCanvas [expr $xk1-32] [expr $yk0+2] 24 24 "proj-$id projExpand projFG" $taskCollapsedIcon
	} else {
	    ptvDrawIconBox $ptvCanvas [expr $xk1-32] [expr $yk0+2] 24 24 "proj-$id projExpand projFG" $taskExpandedIcon
	}
	ptvDrawIconBox $ptvCanvas [expr $xk1-62] [expr $yk0+2] 24 24 "proj-$id projModify projFG" $taskModifyIcon
	ptvDrawIconBox $ptvCanvas [expr $xk1-92] [expr $yk0+2] 24 24 "proj-$id projMemo projFG" $memoIcon
	if { $projWinSize >0 } {incr yk0 34 }
    } elseif { $projWinSize < 4 } {
	ptvDrawIconBox $ptvCanvas [expr $xk0+2] [expr $yk0+2] 16 16 "proj-$id projStatus projFG" $todoStatIcon($stat-small)
	ptvDrawIconBox $ptvCanvas [expr $xk0+22] [expr $yk0+2] 16 16 "proj-$id projDring projFG" $dringIcon($dring-small)
	if { $projExpanded($id) == 0 } {
	    ptvDrawIconBox $ptvCanvas [expr $xk1-22] [expr $yk0+2] 16 16 "proj-$id projExpand projFG" $taskCollapsedIcon16
	} else {
	    ptvDrawIconBox $ptvCanvas [expr $xk1-22] [expr $yk0+2] 16 16 "proj-$id projExpand projFG" $taskExpandedIcon16
	}
	ptvDrawIconBox $ptvCanvas [expr $xk1-43] [expr $yk0+2] 16 16 "proj-$id projModify projFG" $taskModifySmallIcon
	ptvDrawIconBox $ptvCanvas [expr $xk1-64] [expr $yk0+2] 16 16 "proj-$id projMemo projFG" $memoIcon16
	incr yk0 24
    } elseif { $projWinSize < 5 } {
	ptvDrawIconBox $ptvCanvas [expr $xk0+2] [expr $yk0+2] 16 16 "proj-$id projStatus projFG" $todoStatIcon($stat-small)
	if { $projExpanded($id) == 0 } {
	    ptvDrawIconBox $ptvCanvas [expr $xk1-22] [expr $yk0+2] 16 16 "proj-$id projExpand projFG" $taskCollapsedIcon16
	} else {
	    ptvDrawIconBox $ptvCanvas [expr $xk1-22] [expr $yk0+2] 16 16 "proj-$id projExpand projFG" $taskExpandedIcon16
	}
	ptvDrawIconBox $ptvCanvas [expr $xk1-43] [expr $yk0+2] 16 16 "proj-$id projModify projFG" $taskModifySmallIcon
	incr yk0 24
    } elseif { $projWinSize < 7 } {
	ptvDrawIconBox $ptvCanvas [expr $xk0+2] [expr $yk0+2] 16 16 "proj-$id projStatus projFG" $todoStatIcon($stat-small)
	if { $projWinSize < 6 } {
	    if { $projExpanded($id) == 0 } {
		ptvDrawIconBox $ptvCanvas [expr $xk1-22] [expr $yk0+2] 16 16 "proj-$id projExpand projFG" $taskCollapsedIcon16
	    } else {
		ptvDrawIconBox $ptvCanvas [expr $xk1-22] [expr $yk0+2] 16 16 "proj-$id projExpand projFG" $taskExpandedIcon16
	    }
	}

	if { $projWinSize < 5 } {
	    ptvDrawIconBox $ptvCanvas [expr $xk1-43] [expr $yk0+2] 16 16 "proj-$id projModify projFG" $taskModifySmallIcon
	}
	incr yk0 22
    } 


    if { $projWinSize == 0 } {
	$ptvCanvas create rectangle [expr $xk0+64] [expr $yk0 +2 ] [expr $xk1-94] [expr $yk0 + 31] \
	    -fill $bgcol -outline $fgcol -width 1 -tag "proj-$id projFG projTitel"
	set twidth [expr ($xk1-94) - ($xk0+64) -4 ]
	set titel [ strTrunc3 $title $twidth $bfnt16 [string length $title] ]
	$ptvCanvas create text [expr $xk0 + 68 ] [expr $yk0 +28 ] -text $titel \
	    -anchor sw -justify left -font $bfnt16 -tag "proj-$id projFG projTitel"
	incr yk0 34
    } elseif { $projWinSize <3 } {
	$ptvCanvas create rectangle [expr $xk0+4] [expr $yk0 +2 ] [expr $xk1-4] [expr $yk0 + 31] \
	    -fill $bgcol -outline $fgcol -width 1 -tag "proj-$id projFG projTitel"
	set twidth [expr ($xk1-4) - ($xk0+4) -4 ]
	set titel [ strTrunc3 $title $twidth $bfnt16 [string length $title] ]
	$ptvCanvas create text [expr $xk0 + 8 ] [expr $yk0 +28 ] -text $titel \
	    -anchor sw -justify left -font $bfnt16 -tag "proj-$id projFG projTitel"
	incr yk0 34
    } elseif { $projWinSize <4 } {
	$ptvCanvas create rectangle [expr $xk0+4] [expr $yk0 +2 ] [expr $xk1-4] [expr $yk0 + 21] \
	    -fill $bgcol -outline $fgcol -width 1 -tag "proj-$id projFG projTitel"
	set twidth [expr ($xk1-4) - ($xk0+4) -4 ]
	set titel [ strTrunc3 $title $twidth $bfnt12 [string length $title] ]
	$ptvCanvas create text [expr $xk0 + 8 ] [expr $yk0 +20 ] -text $titel \
	    -anchor sw -justify left -font $bfnt12 -tag "proj-$id projFG projTitel"
	incr yk0 24
    } elseif { $projWinSize <5 } {
	$ptvCanvas create rectangle [expr $xk0+4] [expr $yk0 +2 ] [expr $xk1-4] [expr $yk0 + 15] \
	    -fill $bgcol -outline $fgcol -width 1 -tag "proj-$id projFG projTitel"
	set twidth [expr ($xk1-4) - ($xk0+4) -4 ]
	set titel [ strTrunc3 $title $twidth $bfnt10 [string length $title] ]
	$ptvCanvas create text [expr $xk0 + 8 ] [expr $yk0 +14 ] -text $titel \
	    -anchor sw -justify left -font $bfnt10 -tag "proj-$id projFG projTitel"
	incr yk0 16
    }
    

    if { $projWinSize == 0 } {
	$ptvCanvas create text [expr $xk0 +4] [expr $yk0 +2]  -text "$_projText(projDescr):"\
	    -anchor nw -justify left -font $fnt -tag "proj-$id projFG projDescr projDescr-$id"
	$ptvCanvas create text [expr $xk0 +8 + [font measure $fnt "$_projText(projDescr):"] ]  [expr $yk0 +2]\
	    -text $descr -width [expr $mwidth -4] \
	    -anchor nw -justify left -font $fnt -tag "proj-$id projFG projDescr projDescr-$id"
    } elseif { $projWinSize <3 } {
	$ptvCanvas create text [expr $xk0 +8]  [expr $yk0 +2] -text $descr -width [expr $mwidth +56] \
	    -anchor nw -justify left -font $fnt -tag "proj-$id projFG projDescr projDescr-$id"
	set yk1 [expr  [lindex [$ptvCanvas bbox projDescr-$id ] 3 ] +2 ]
	set dy [expr $yk1 - $yk0]
	
	set dt $descr
	set l [string length $dt]
	while { $dy > 64 } {
	    incr l -1
	    if { $l < 8 }  {
		break
	    }
	    set dt [string range $dt 0 $l ]
	    $ptvCanvas itemconfigure projDescr-$id -text "$dt ..."
	    set yk1 [expr  [lindex [$ptvCanvas bbox projDescr-$id ] 3 ] +2 ]
	    set dy [expr $yk1 - $yk0]
	}
    }
    if { $projWinSize <3 } {
	set yk1 [expr  [lindex [$ptvCanvas bbox projDescr-$id ] 3 ] +2 ]
	$ptvCanvas create rectangle [expr $xk0+2] $yk0 [expr $xk1-2] $yk1 \
	    -fill $bgcol -outline $fgcol   -tag "proj-$id projFG  projDescrBG-$id"
	$ptvCanvas lower projDescrBG-$id projDescr-$id
    } else {
	set yk1 $yk0

    }
    $ptvCanvas create rectangle [expr $xk0-2 ] [expr $ykt -1 ] [expr $xk1 +2 ] [expr $yk1 + 4] \
	-fill $bgcol -outline $fgcol -width 3 -tag "proj-$id projBG"
    



    set yk0 $yk1
    incr yk0 6
    incr yk0 6
    if { $projExpanded($id) == 1 } {
	foreach ent [lindex $tdd_projList($id) 12 ] {
	    set sid  [lindex $ent 0 ]
	    set taskHidden($sid) 0
	    if { $sid < 0 } {
		set  yk0 [ptvDrawProj $yk0 [expr -1*$sid ] ]
	    } else {
		set  yk0 [ptvDrawTask $yk0 $sid ]
	    }
	    incr yk0 24
	}
	incr yk0 -16
    } else {
	foreach ent [lindex $tdd_projList($id) 12 ] {
	    set sid  [lindex $ent 0 ]
	    set taskHidden($sid) 1
	}
    }
    if { $projWinSize <4  } {
	set yk0 [ptvDrawProjStatusBox  $xk0 $yk0 $xk1 $id $fgcol $bgcol]
    } elseif {$projWinSize <6 } {
	set yk0 [ptvDrawProjCompl  $xk0 $yk0 $xk1 $id $fgcol $bgcol]
    } else {
	set yk0 [ptvDrawEmptyProjStatusBox  $xk0 $yk0 $xk1 $id $fgcol $bgcol]
	
    }   


    if {    $ptvProjLeftOpen  == 1 } {
	$ptvCanvas create line  [expr $xk0-2 ] [expr $ykt -1 ] [expr $xk0-2 ] [expr $yk0 +2 ] \
	    -fill $fgcol -width 3 -tag "proj-$id projBG projBG-$id" -dash {5 12}
    } else { 
		$ptvCanvas create line  [expr $xk0-2 ] [expr $ykt -1 ] [expr $xk0-2 ] [expr $yk0 +2] \
	    -fill $fgcol -width 3 -tag "proj-$id projBG projBG-$id"
    }
    if {    $ptvProjRightOpen  == 1 } {
	$ptvCanvas create line  [expr $xk1+2 ] [expr $ykt -1 ] [expr $xk1+2 ] [expr $yk0 +2 ] \
	    -fill $fgcol -width 3 -tag "proj-$id projBG projBG-$id" -dash {4 8}
    } else { 
	$ptvCanvas create line  [expr $xk1+2 ] [expr $ykt -1 ] [expr $xk1+2 ] [expr $yk0 +2 ] \
	    -fill $fgcol -width 3 -tag "proj-$id projBG projBG-$id"
    }
    $ptvCanvas create line  [expr $xk0-2 ] [expr $ykt -1 ] [expr $xk1+2 ] [expr $ykt -1 ] \
	-fill $fgcol -width 3 -tag "proj-$id projBG projBG-$id"
    $ptvCanvas create line  [expr $xk0-2 ] [expr $yk0  ] [expr $xk1+2 ] [expr $yk0 ] \
	-fill $fgcol -width 3 -tag "proj-$id projBG projBG-$id"

    $ptvCanvas lower projBG  taskBG
    $ptvCanvas lower projBG projFG

    if { $projExpanded($id) == 1 } {
	foreach ent [lindex $tdd_projList($id) 12 ] {
	    set toID [lindex $ent 0]
	    set depList [lindex $ent 1]
#	    puts "DEP for $toID  //$depList//"
	    foreach dep $depList {
#		puts "\t\tdep=//$dep//"
		ptvDrawDep $toID [lindex $dep 0] [lindex $dep 1]  [lindex $dep 2] 
	    }
	}
    }
    return $yk0
}

proc ptvDrawProjStatusBox { xk0 yk0 xk1 id fgcol bgcol } {
    global ptvCanvas todoStatIcon  mileStoneTranspIcon
    global ptvDepSource fnt10 fnt8 bfnt10
    global tdd_projList
    global ptvProjDepSourceX  ptvProjDepSourceY ptvProjDepSourceCount



    set list [ $ptvCanvas bbox proj-$id]
    # -4 for the missing windowframe
    set sx [expr  [ lindex $list 0] -4 ]
    set sy [ lindex $list 1]


    $ptvCanvas create rectangle [expr $xk0  ] [expr $yk0   ] [expr $xk1  ] [expr $yk0 + 40]\
	-fill $bgcol  -outline $bgcol  -tag "proj-$id projFG  projStatusBox-$id"
    $ptvCanvas create line  [expr $xk0] [expr $yk0 ] [expr $xk1] [expr $yk0 ]\
	-fill $fgcol  -width 3 -tag "proj-$id projFG  projStatusBox-$id"


    $ptvCanvas create rectangle [expr $xk0+2] [expr $yk0 +4] [expr $xk1-2] [expr $yk0 + 38]\
	-fill $bgcol  -outline $fgcol  -tag "proj-$id projFG  projStatusBox-$id"

    set mwidth [ expr ($xk1 -6) - ($xk0 +6) ]

    ptvDrawIconBox $ptvCanvas [expr $xk1-24] [expr $yk0+4] 12 12 "proj-$id projStatusBox-$id projDepDONE projFG" $todoStatIcon(DONE-small)
    ptvDrawIconBox $ptvCanvas [expr $xk1-44] [expr $yk0+4] 12 12 "proj-$id projStatusBox-$id projDepKILL projFG" $todoStatIcon(KILL-small)
    ptvDrawIconBox $ptvCanvas [expr $xk1-64] [expr $yk0+4] 12 12 "proj-$id projStatusBox-$id projDepHOLD projFG" $todoStatIcon(HOLD-small)
    ptvDrawIconBox $ptvCanvas [expr $xk1-84] [expr $yk0+4] 12 12 "proj-$id projStatusBox-$id projDepDELEY projFG" $todoStatIcon(DELAY-small)
    set x [expr $xk0 + int(floor($mwidth/2))-6]
    if { $x >  [expr $xk1-104] } { set x [expr $xk1-108] }
    ptvDrawIconBox $ptvCanvas $x [expr $yk0+4] 12 12 "proj-$id projStatusBox-$id projDepWIP projFG" $todoStatIcon(WIP-small)
    set ptvProjDepSourceX($id-ONDONE)  [expr $xk1-18 - $sx]
    set ptvProjDepSourceX($id-ONKILL)  [expr $xk1-38 - $sx]
    set ptvProjDepSourceX($id-ONHOLD)  [expr $xk1-58 - $sx]
    set ptvProjDepSourceX($id-ONDELAY) [expr $xk1-78 - $sx]
    set ptvProjDepSourceX($id-ONWIP)   [expr $x - $sx]
    set ptvProjDepSourceY($id-ONDONE)  [expr $yk0+23 - $sy]
    set ptvProjDepSourceY($id-ONKILL)  [expr $yk0+23 - $sy]
    set ptvProjDepSourceY($id-ONHOLD)  [expr $yk0+23 - $sy]
    set ptvProjDepSourceY($id-ONDELAY) [expr $yk0+23 - $sy]
    set ptvProjDepSourceY($id-ONWIP)   [expr $yk0+23 - $sy]
    set ptvProjDepSourceCount($id) 0

    set compl      [lindex $tdd_projList($id) 11 ]

    $ptvCanvas create rectangle [expr $xk0+6] [expr $yk0 +23] [expr $xk1-6] [expr $yk0 + 36]\
	-fill "#704C0A" -outline black -tag "proj-$id projFG projStatusBox-$id projComplete" 
    $ptvCanvas create rectangle [expr $xk0+6] [expr $yk0 +23] [expr $xk0+6 + int(floor($mwidth*$compl/100.))] [expr $yk0 + 36]\
	-fill "#FFAE1A" -outline black -tag "proj-$id projFG projStatusBox-$id projComplete" 
    set ptvProjDepSourceX($id-ONCOMPL)  "[expr $xk0 +6 -  $sx] $mwidth"
    set ptvProjDepSourceY($id-ONCOMPL)   [expr $yk0+38 - $sy]

    return [expr $yk0 + 42 ]
}

proc ptvDrawProjCompl {xk0 yk0 xk1 id fgcol bgcol } {
    global ptvProjDepSourceX ptvProjDepSourceY ptvCanvas tdd_projList  ptvProjDepSourceCount

    set compl      [lindex $tdd_projList($id) 11 ]
    set mwidth [ expr ($xk1 -3) - ($xk0 +3) ]
    set list [ $ptvCanvas bbox proj-$id]
    # -4 for the missing windowframe
    set sx [expr  [ lindex $list 0] -4 ]
    set sy [ lindex $list 1]

    $ptvCanvas create rectangle [expr $xk0  ] [expr $yk0   ] [expr $xk1  ] [expr $yk0 + 20]\
	-fill $bgcol  -outline $bgcol  -tag "proj-$id projFG  projStatusBox-$id"
    $ptvCanvas create line  [expr $xk0] [expr $yk0 ] [expr $xk1] [expr $yk0 ]\
	-fill $fgcol  -width 3 -tag "proj-$id projFG  projStatusBox-$id"

    $ptvCanvas create rectangle [expr $xk0+3] [expr $yk0 +4] [expr $xk1-3] [expr $yk0 + 17]\
	-fill "#704C0A" -outline black -tag "proj-$id projFG projStatusBox-$id projComplete" 
    $ptvCanvas create rectangle [expr $xk0+3] [expr $yk0 +4] [expr $xk0+3 + int(floor($mwidth*$compl/100.))] [expr $yk0 + 17]\
	-fill "#FFAE1A" -outline black -tag "proj-$id projFG projStatusBox-$id projComplete" 
    set ptvProjDepSourceX($id-ONCOMPL)  "[expr $xk0 +6 -  $sx] $mwidth"
    set ptvProjDepSourceY($id-ONCOMPL)   [expr $yk0+19 - $sy]

    set ptvProjDepSourceX($id-ONDONE)  [expr $xk1-2 - $sx]
    set ptvProjDepSourceX($id-ONKILL)  [expr $xk1-2 - $sx]
    set ptvProjDepSourceX($id-ONHOLD)  [expr $xk1-2 - $sx]
    set ptvProjDepSourceX($id-ONDELAY) [expr $xk1-2 - $sx]
    set ptvProjDepSourceX($id-ONWIP)   [expr $xk1-2 - $sx]
    set ptvProjDepSourceY($id-ONDONE)  [expr $yk0+23 - $sy]
    set ptvProjDepSourceY($id-ONKILL)  [expr $yk0+23 - $sy]
    set ptvProjDepSourceY($id-ONHOLD)  [expr $yk0+23 - $sy]
    set ptvProjDepSourceY($id-ONDELAY) [expr $yk0+23 - $sy]
    set ptvProjDepSourceY($id-ONWIP)   [expr $yk0+23 - $sy]
    set ptvProjDepSourceCount($id) 0
#    puts "set depcount($id) = $ptvProjDepSourceCount($id)"


    return [expr $yk0 + 20 ]
}
proc ptvDrawEmptyProjStatusBox {xk0 yk0 xk1 id fgcol bgcol } {
    global ptvProjDepSourceX ptvProjDepSourceY ptvCanvas tdd_projList  ptvProjDepSourceCount

    set compl      [lindex $tdd_projList($id) 11 ]
    set mwidth [ expr ($xk1 -3) - ($xk0 +3) ]
    set list [ $ptvCanvas bbox proj-$id]
    # -4 for the missing windowframe
    set sx [expr  [ lindex $list 0] -4 ]
    set sy [ lindex $list 1]

    $ptvCanvas create rectangle [expr $xk0  ] [expr $yk0   ] [expr $xk1  ] [expr $yk0 + 4]\
	-fill $bgcol  -outline $bgcol  -tag "proj-$id projFG  projStatusBox-$id"
    $ptvCanvas create line  [expr $xk0] [expr $yk0 ] [expr $xk1] [expr $yk0 ]\
	-fill $fgcol  -width 3 -tag "proj-$id projFG  projStatusBox-$id"

    set ptvProjDepSourceX($id-ONCOMPL)  "[expr $xk1-2 -$sx] [expr $xk1-2 -$sx]"
    set ptvProjDepSourceY($id-ONCOMPL)   [expr $yk0+23 - $sy]

    set ptvProjDepSourceX($id-ONDONE)  [expr $xk1-2 - $sx]
    set ptvProjDepSourceX($id-ONKILL)  [expr $xk1-2 - $sx]
    set ptvProjDepSourceX($id-ONHOLD)  [expr $xk1-2 - $sx]
    set ptvProjDepSourceX($id-ONDELAY) [expr $xk1-2 - $sx]
    set ptvProjDepSourceX($id-ONWIP)   [expr $xk1-2 - $sx]
    set ptvProjDepSourceY($id-ONDONE)  [expr $yk0+23 - $sy]
    set ptvProjDepSourceY($id-ONKILL)  [expr $yk0+23 - $sy]
    set ptvProjDepSourceY($id-ONHOLD)  [expr $yk0+23 - $sy]
    set ptvProjDepSourceY($id-ONDELAY) [expr $yk0+23 - $sy]
    set ptvProjDepSourceY($id-ONWIP)   [expr $yk0+23 - $sy]
    set ptvProjDepSourceCount($id) 0
#    puts "set depcount($id) = $ptvProjDepSourceCount($id)"
    return [expr $yk0 + 4 ]
}

proc ptvDrawDep { to from type val } {
    global ptvTaskDepSourceX  ptvTaskDepSourceY ptvTaskDepSourceCount
    global ptvProjDepSourceX  ptvProjDepSourceY ptvProjDepSourceCount
    global ptvCanvas  ptvDayWidth
    global tdd_todoList  tdd_projList
    global ptvDateCoords ptvStartDate

    puts "DEP $from  $type $val -> $to"

    if { $from < 0 } {
	set src [expr -$from]
	set fromList  [$ptvCanvas bbox projBG-$src ]
	set stag "proj"
    } else {
	set src $from
	set fromList [$ptvCanvas bbox taskBG-$src ]
#	puts "IDX: [$ptvCanvas find withtag taskBG-$src] ($src) fromList=/$fromList/"
	puts "DEP: fromList:/$fromList/"
	set stag "task"
    }
    if { $to < 0 } {
	set dst [expr -$to]
	set toList [$ptvCanvas bbox proj-$dst ]
	set dtag "proj"
    } else {
	set dst $to
	set toList [$ptvCanvas bbox task-$dst ]
	set dtag "task"
    }
    
#    puts "DEP: $fromList / $stag   -> /$toList/ $dtag"

    if { $fromList == "" && $toList == "" } { return }
    if { $fromList != "" } {
	set fx0 [lindex $fromList 0]
	set fx1 [lindex $fromList 2]
	set fy0 [lindex $fromList 1]
	set fy1 [lindex $fromList 3]
    }
    
    if { $toList != "" } {
	set tx0 [lindex $toList 0]
	set ty0 [lindex $toList 1]
    } 
    

    set done 0
    if { $from < 0 } { 
	if  { $type == "ONCOMPL" } {
	    if { [lindex $tdd_projList($src) 11] >= $val } { set done 1} 
	} else {
	    set stat [ lindex $tdd_projList($src) 2]
	    if { [ string range $type 2 end] == $stat } { set done 1}
	}
    } else {
	if { $type == "ONMILES" } {
	    set miles [ lindex $tdd_todoList($src) 12]
	    set ms [lindex $miles $val ]
	    if { [lindex $ms 0] == 1 } { set done 1}
	} elseif  { $type == "ONCOMPL" } {
	    if { [lindex $tdd_todoList($src) 6] >= $val } { set done 1}
	} else {
	    set stat [ lindex $tdd_todoList($src) 2]
	    if { [ string range $type 2 end] == $stat } { set done 1}
	}
    }
    if { $done == 1 } {
	set color green4
	set etag "dep-done"
    } else {
	set color red2 
	set etag "dep-open"
    }

    if { $fromList == "" } {
	set dy [expr $ty0 + 12 ]
	$ptvCanvas create line  3 $dy $tx0 $dy -arrow last -fill $color -width 3 \
	    -tag "Dep-$stag-$src-$dtag-$dst-$type-$val dep $etag" 
    } else {
	if { $from < 0 } { 
	    if  { $type == "ONCOMPL" } {
		set sx [expr [lindex $ptvProjDepSourceX($src-$type) 0] + int(floor(double([ lindex $ptvProjDepSourceX($src-$type) 1]*$val)/100.))  + $fx0 ]
	    } else {
		set sx [expr $ptvProjDepSourceX($src-$type) + $fx0 ]
	    }
	    set sy [expr $fy1 + $ptvProjDepSourceCount($src) *4+2]
	    set sy0 [expr $ptvProjDepSourceY($src-$type) + $fy0 ]
	    incr ptvProjDepSourceCount($src)
#	    puts "count($src - $type) = $ptvProjDepSourceCount($src)"
	    #	puts "count = $ptvProjDepSourceCount($src)"
	} else {
	    if { $type == "ONMILES" } {
		set tx [lindex $ptvTaskDepSourceX($src-$type) $val ]
		set sx [expr $tx + $fx0 ]
	    } elseif  { $type == "ONCOMPL" } {
		set sx [expr [lindex $ptvTaskDepSourceX($src-$type) 0] + int(floor(double([ lindex $ptvTaskDepSourceX($src-$type) 1]*$val)/100.))  + $fx0 ]
	    } else {
		set sx [expr $ptvTaskDepSourceX($src-$type) + $fx0 ]
	    }
	    set sy [expr $fy1 + $ptvTaskDepSourceCount($src) *4+2]
	    set sy0 [expr $ptvTaskDepSourceY($src-$type) + $fy0 ]
	    puts "\t\t\tptvTaskDepSourceY($src-$type) = $ptvTaskDepSourceY($src-$type)"
	    incr ptvTaskDepSourceCount($src)
#	    puts "Count($src - $type) = $ptvTaskDepSourceCount($src)"
	}




	set dx0 [expr int(ceil(0.75* $ptvDayWidth)) ]
	if { $dx0 < 12 } { set dx0 12 }

	if { $toList == "" } {
	    set tx  [expr $sx + $dx0 ]
	    $ptvCanvas create line $sx $fy1   $sx $sy    -arrow last -fill $color -width 3 \
		    -tag "Dep-$stag-$src-$dtag-$dst-$type-$val dep $etag" 
	} else { 
	    set dx [expr $tx0 - $dx0 ]
	    if { $dx <2 } { set dx 2 }
	    set dy [expr $ty0 + 12 ]
	    puts "DEP dx=$dx sx=$sx $ptvDateCoords($ptvStartDate) $ptvStartDate"
	    if { $sx > $dx } {
		puts "line $sx $fy1   $sx $sy   $dx $sy   $dx $dy   $tx0 $dy  ($ty0)"
		$ptvCanvas create line $sx $fy1   $sx $sy   $dx $sy   $dx $dy   $tx0 $dy -arrow last \
		    -fill $color -width 3 \
		    -tag "Dep-$stag-$src-$dtag-$dst-$type-$val dep $etag" 
	    } else {
		$ptvCanvas create line $sx $fy1  $sx $dy $tx0 $dy -arrow last -fill $color -width 3 \
		    -tag "Dep-$stag-$src-$dtag-$dst-$type-$val dep $etag" 
	    }
	}
	#	puts "fromList=>$fromList<"
	if { $fromList != "" } {
	    puts "line2 : $sx $sy0 $sx $fy1"
	    set tags [format "Dep-%s-%s-%s-%s-%s-%s  %sFG  depInTask %s-%s %sStatusBox-%s"  \
			  $stag $src $dtag $dst $type $val  $stag  $stag $src $stag $src   ]
	    $ptvCanvas create line $sx $sy0 $sx $fy1 -fill $color -width 3 -tag $tags
	}
    }
}


proc ptvCheckRemoveDep { x y} {
    global ptvCanvas tdd_todoPID ptvCurrentDep
    global fnt tdd_todoList tdd_projList

#    puts " dep for [$ptvCanvas gettags current]  @ $x $y "
    set list [$ptvCanvas gettags current]
    set p [lsearch $list "Dep-*" ]
    if { $p < 0  } { return }
    set dep [lindex $list $p]
#    puts $dep

    set list [split $dep -]
    if { [lindex $list 1] =="task" } {
	set src [lindex $list 2]
	set srcStr "der Aufgabe \"[lindex $tdd_todoList($src) 3]\""
    } else {
	set src "-[lindex $list 2]"
	set srcStr "des Projekts \"[lindex $tdd_projList([lindex $list 2]) 5]\""
    }
    if { [lindex $list 3] =="task" } {
	set dest [lindex $list 4]
	set destStr "der Aufgabe \"[lindex $tdd_todoList($dest) 3]"
    } else {
	set dest "-[lindex $list 4]"
	set destStr "dem Projekt \"[lindex $tdd_projList([lindex $list 4]) 5]\""
    }

    set ptvCurrentDep "$src $dest [lindex $list 5] [lindex $list 6] $dep"

    toplevel .crd 
    frame .crd.t -relief raised -bd 1
    frame .crd.b -relief flat
    pack .crd.t .crd.b -side top -fill x

    message .crd.t.t -text "Wollen Sie wirklich die Abhnggkeit $destStr von $srcStr entfernen?" -width 240 -font $fnt
    pack .crd.t.t -side top -fill x

    button .crd.b.ok -text " O.K. " -command {
	destroy .crd
	ptvRemoveDep
	set ptvCurrentDep ""
    }
    button .crd.b.abb -text "Abbruch" -command {
	destroy .crd
	set ptvCurrentDep ""
    }
    pack .crd.b.ok -side left 
    pack .crd.b.abb -side right
    wm title .crd "Abhnggkeit entfernen?"
    #centerWin .crd  260 108
#    update
#    puts [wm geometry .crd]
    wm geometry .crd +$x+$y
    grab .crd

}
proc ptvRemoveDep { } {
    global ptvCanvas  ptvCurrentDep tdd_todoPID

    set parent $tdd_todoPID([lindex $ptvCurrentDep 1])
    modifyDep del $parent  [lindex $ptvCurrentDep 0] [lindex $ptvCurrentDep 1] [lindex $ptvCurrentDep 2] [lindex $ptvCurrentDep 3]
    $ptvCanvas delete  [lindex $ptvCurrentDep 4]
}

proc ptvDrawTask { y id} { 
    #{stat titel start stop complete users init dring descr mileStones} { }
    global ptvDayWidth ptvDateCoords ptvCanvas ptvTaskCoord ptvTaskWindSize
    global todoStatIcon dringIcon quitIcon taskExpandedIcon taskCollapsedIcon taskModifySmallIcon taskModifyIcon
    global memoIcon memoIcon16 taskModifyIcon16 taskExpandedIcon16  taskCollapsedIcon16
    global projColorRingBG ptvProjColorID
    global bfnt16 bfnt fnt10 bfnt12
    global tdd_todoList tdd_todoPID

    set stat    [lindex $tdd_todoList($id) 2]
    set title   [lindex $tdd_todoList($id) 3]
    set start   [lindex $tdd_todoList($id) 4]
    set stop    [lindex $tdd_todoList($id) 5]
    set dring   [lindex $tdd_todoList($id) 9]

    global ptvStartDate ptvStopDate

#    puts "Task $id : $title  @ $start - $stop ($ptvStartDate ...  $ptvStopDate)"
    if { $start > $ptvStopDate || $stop <= $ptvStartDate } { return $y}
    set ptvTaskLeftOpen($id) 0
    set ptvTaskRightOpen($id) 0
    if { $start >= $ptvStartDate } {
	set xk0 $ptvDateCoords($start)
    } else { 
	set xk0 $ptvDateCoords($ptvStartDate)
	set ptvTaskLeftOpen($id) 1
    }
    if { $stop <= $ptvStopDate } {
	set xk1 $ptvDateCoords($stop)
    } else { 
	set xk1 $ptvDateCoords($ptvStopDate)
	set ptvTaskRightOpen($id) 1
    }

    set dx [expr  $xk1 -$xk0 ] 
    if { $dx >= 260 } { set ptvTaskWindSize($id) 0 
    } elseif { $dx >= 172 } {set ptvTaskWindSize($id) 1
    } elseif { $dx >= 136 } {set ptvTaskWindSize($id) 2
    } elseif { $dx >= 105 } {set ptvTaskWindSize($id) 3
    } elseif { $dx >=  60 } {set ptvTaskWindSize($id) 4
    } elseif { $dx >=  42 } {set ptvTaskWindSize($id) 5
    } elseif { $dx >=  22 } {set ptvTaskWindSize($id) 6
    } else { set ptvTaskWindSize($id) 7 }

    puts "WInSIZE($id)=$ptvTaskWindSize($id) <- $dx // '$title'"

    set yk0 $y
    set ykt $y
    incr xk0
    incr xk1 -1
    set ptvTaskCoord($id) $yk0


    #The Icons
    if { $ptvTaskWindSize($id) <=3 } {
	ptvDrawIconBox $ptvCanvas [expr $xk0+2]  [expr $yk0+2] 16 16 "task-$id taskStatus taskFG" \
	    $todoStatIcon($stat-small)
	ptvDrawIconBox $ptvCanvas [expr $xk0+23] [expr $yk0+2] 16 16 "task-$id taskDring taskFG"  \
	    $dringIcon($dring-small)
    }
    if { $ptvTaskWindSize($id) <=1 } {
	ptvDrawIconBox $ptvCanvas [expr $xk1-65] [expr $yk0+2] 16 16 "task-$id taskMemo taskFG" \
	    $memoIcon16
    }
    if { $ptvTaskWindSize($id) <=2 } {
	ptvDrawIconBox $ptvCanvas [expr $xk1-44] [expr $yk0+2] 16 16 "task-$id taskModify taskFG" \
	    $taskModifyIcon16
	ptvDrawIconBox $ptvCanvas [expr $xk1-23] [expr $yk0+2] 16 16 "task-$id taskExpand taskFG taskExpand-$id" \
	    $taskCollapsedIcon16
    } elseif { $ptvTaskWindSize($id) ==3  } {
	ptvDrawIconBox $ptvCanvas [expr $xk1-23] [expr $yk0+2] 16 16 "task-$id taskModify taskFG" \
	    $taskModifySmallIcon
    } elseif { $ptvTaskWindSize($id) >=4  && $ptvTaskWindSize($id) <=6} {
	ptvDrawIconBox $ptvCanvas [expr $xk0+2]  [expr $yk0+2] 12 12  "task-$id taskStatus taskFG" \
	    $todoStatIcon($stat-small)
	if { $ptvTaskWindSize($id) ==4   } {
	    ptvDrawIconBox $ptvCanvas [expr $xk0+20]  [expr $yk0+2] 12 12  "task-$id taskDring taskFG" \
		$dringIcon($dring-small)
	}
	if { $ptvTaskWindSize($id) <6   } {
	    ptvDrawIconBox $ptvCanvas [expr $xk1-20]  [expr $yk0+2] 12 12  "task-$id taskModify taskFG" \
		$taskModifySmallIcon
	}
    }

    #The Title
    switch $ptvTaskWindSize($id) {
	0 { set twidth [expr ($xk1-65) - ($xk0 +44) -4 ] }
	1 -
	2 -
	3 -
	4 { set twidth [expr $xk1 - $xk0  - 7 ] }
	5 -
	6  { set twidth 0}
    }
    if { $ptvTaskWindSize($id)  <=3 } {
	set titel [ strTrunc3 $title $twidth $bfnt16 [string length $title] ]
    } elseif { $ptvTaskWindSize($id)  ==4 } {
	set titel [ strTrunc3 $title $twidth $bfnt12 [string length $title] ]
    }
    if { $ptvTaskWindSize($id)  == 0 } {
	$ptvCanvas create rectangle [expr $xk0+46] [expr $yk0 +2 ] [expr $xk1-67] [expr $yk0 + 21 ] \
	    -fill $projColorRingBG($ptvProjColorID($tdd_todoPID($id))) -outline black \
	    -tag "task-$id taskFG taskTitel taskTitel-$id"
	$ptvCanvas create text [expr $xk0 + 48 ] [expr $yk0 +20 ] -text $titel \
	    -anchor sw -justify left -font $bfnt16 -tag "task-$id taskFG taskTitel taskTitel-$id"
	incr yk0 32
    } elseif  { $ptvTaskWindSize($id) <=3 } {
	$ptvCanvas create rectangle [expr $xk0+3] [expr $yk0 + 23 ] [expr $xk1-4] [expr $yk0 + 42 ] \
	    -fill $projColorRingBG($ptvProjColorID($tdd_todoPID($id))) -outline black \
	    -tag "task-$id taskFG taskTitel taskTitel-$id"
	$ptvCanvas create text [expr $xk0 + 5 ] [expr $yk0 +41 ] -text $titel \
	    -anchor sw -justify left -font $bfnt16 -tag "task-$id taskFG taskTite taskTitel-$id"
	incr yk0 69
    } elseif  { $ptvTaskWindSize($id) ==4 } {
	$ptvCanvas create rectangle [expr $xk0+2] [expr $yk0 + 21 ] [expr $xk1-2] [expr $yk0 + 35 ] \
	    -fill $projColorRingBG($ptvProjColorID($tdd_todoPID($id))) -outline black \
	    -tag "task-$id taskFG taskTitel taskTitel-$id"
	$ptvCanvas create text [expr $xk0 + 3 ] [expr $yk0 +35 ] -text $titel \
	    -anchor sw -justify left -font $bfnt12 -tag "task-$id taskFG taskTitel taskTitel-$id"
	incr yk0 38
    } else {
	incr yk0 22
    }
    if { $ptvTaskWindSize($id) <4 } {
	ptvDrawTaskSmall $xk0 $xk1 $id
    }

    if { $ptvTaskWindSize($id) <4 } {
	set yk0 [lindex [ $ptvCanvas bbox taskVariable-$id]  3]    

	set yk1 [ptvDrawTaskStatusBox $xk0 [expr $yk0 + 0] $xk1 $id ]
	incr yk1 +3
    } else {
	ptvDrawTaskEmptyStatusBox $xk0 [expr $yk0 + 0] $xk1 $id
	set yk1 $yk0
    }

    ptvDrawRelief $ptvCanvas $xk0 $ykt $xk1 $yk1 "task-$id taskBG taskBG-$id" grey86
    $ptvCanvas lower grid  task-$id 
    $ptvCanvas raise taskFG taskBG
    return $yk1
}

proc ptvDrawTaskSmall { xk0 xk1  id } {
    global ptvCanvas  fnt10
    global ptvTaskExpand
    global tdd_todoList colorList _projText


    set yk0 [lindex [ $ptvCanvas bbox taskTitel-$id]  3]
    set users [lindex $tdd_todoList($id) 7 ]
    set init  [lindex $tdd_todoList($id) 8 ]

    $ptvCanvas delete taskVariable-$id
    $ptvCanvas create text [expr $xk0 +4] [expr $yk0 + 2] -text "$_projText(ptvWorker) :"\
	-anchor nw -justify left -font $fnt10 -tag "task-$id taskFG taskWorker taskVariable-$id"
    set mwidth [ expr ($xk1 -4) - ($xk0 +64) ]
    $ptvCanvas create rectangle [expr $xk0+64] [expr $yk0 +2 ] [expr $xk1-4] [expr $yk0 + 12]\
	-fill white -outline black -tag "task-$id taskFG taskWorker taskVariable-$id"
#    FIXME Colorcode of users
    set xc0 [expr $xk0+65]
    set xc1 [expr $xc0+ 8]
    set yc0 [expr $yk0+ 3]
    set yc1 [expr $yk0+ 7]
    set yc2 [expr $yk0+11]
    foreach usr $users {
	set uname "u-[lindex $usr 0]"
	set col1 [lindex $colorList($uname) 0]
	set col2 [lindex $colorList($uname) 1]
	$ptvCanvas create rectangle $xc0 $yc0 $xc1 $yc1 \
	    -fill $col1 -outline $col1 -tag  "task-$id taskFG taskWorker taskVariable-$id"
	$ptvCanvas create rectangle $xc0 $yc1 $xc1 $yc2 \
	    -fill $col2 -outline $col2 -tag  "task-$id taskFG taskWorker taskVariable-$id"
	incr xc0 8
	incr xc1 8
	if { $xc1 >  [expr $xk1-4] } { break }
    }
    set ptvTaskExpand($id) 0
#    puts "bbox($id) [$ptvCanvas bbox taskVariable-$id] "
}

proc ptvDrawTaskFull { xk0 xk1  id } {
    global ptvCanvas  fnt10 fnt12
    global ptvTaskExpand
    global tdd_todoList
    global ptvTaskWindSize 
    global fullUserNames colorList _projText

    if { $ptvTaskWindSize($id) >1 } { return }

    set yk0 [lindex [ $ptvCanvas bbox taskTitel-$id]  3]
    set users [lindex $tdd_todoList($id) 7 ]
    set init  [lindex $tdd_todoList($id) 8 ]
    set descr [lindex $tdd_todoList($id) 11]
    set miles [lindex $tdd_todoList($id) 12]

    set bottom  [lindex  [$ptvCanvas bbox taskVariable-$id] 3]
    $ptvCanvas delete taskVariable-$id


    set mwidth [ expr ($xk1 -4) - ($xk0 +64) ]
    if { $ptvTaskWindSize($id) ==0 } { 
	set colWidth 192. 
	set nameWidth 128
	set milesWidth [expr $mwidth -4 -18 -36]
	if { $milesWidth > 256 } { set milesWidth 256}
	set tfnt $fnt12
	set rowHeight 15
    } else { 
	set nameWidth [expr $mwidth -28]
	set milesWidth [expr $mwidth -4 -18]
	set colWidth  [expr $mwidth -4 ]
	set tfnt $fnt10
	set rowHeight 13
    }
    set cols  [ expr int(floor(double($mwidth) / $colWidth)) ]
    set userAnz [ llength $users]
    set rows [ expr int(ceil(double($userAnz)/double($cols))) ]


#    puts "rows=$rows"
    $ptvCanvas create rectangle [expr $xk0+4] [expr $yk0 + 2] [expr $xk1-4] [expr $yk0 +  6 + $rows *$rowHeight]\
	-fill grey96 -outline black -tag "task-$id taskFG  taskVariable-$id"


    $ptvCanvas create text [expr $xk0 +6] [expr $yk0 +  4] -text "$_projText(ptvWorker) :"\
	-anchor nw -justify left -font $fnt10 -tag "task-$id taskFG taskWorker taskVariable-$id"
    set c 0
    set r 0
    foreach usr $users {
	set uname "u-[lindex $usr 0]"
	set name $fullUserNames($uname)
	set res  [lindex $usr 1]

	set x [expr $xk0 + 64 + $c *$colWidth ]
	set y [expr $yk0 +  4 + $r *$rowHeight ]

	#FIXME ColorCode 
	set nam [ nameStrFntTrunc $name $nameWidth $tfnt]
	if {  $ptvTaskWindSize($id) ==0 } { 
	    set col1 [lindex $colorList($uname) 0]
	    set col2 [lindex $colorList($uname) 1]
	    $ptvCanvas create rectangle [expr $x +4] [expr $y +1] [expr $x+16] [expr $y+7 ] \
		-fill $col1 -outline $col1 -tag  "task-$id taskFG taskWorker taskVariable-$id"
	    $ptvCanvas create rectangle [expr $x +4] [expr $y +8] [expr $x+16] [expr $y+13 ] \
		-fill $col2 -outline $col2 -tag  "task-$id taskFG taskWorker taskVariable-$id"
	    $ptvCanvas create text [expr $x + 18] [expr $y+13] -text $nam \
		-anchor sw -justify left -font $tfnt -tag  "task-$id taskFG taskWorker taskVariable-$id"
	    $ptvCanvas create rectangle [expr $x +20+$nameWidth] [expr $y +2] [expr $x+20+$nameWidth+40] [expr $y+11 ] \
		-fill white -outline black -tag  "task-$id taskFG taskWorker taskVariable-$id"
	    $ptvCanvas create rectangle [expr $x +20+$nameWidth] [expr $y +2] [expr $x+20+$nameWidth+int(floor($res/100.*40))] [expr $y+11 ] \
		-fill red2 -outline black -tag  "task-$id taskFG taskWorker taskVariable-$id"
	} else {
	    $ptvCanvas create text [expr $x + 4] [expr $y+11] -text $nam \
		-anchor sw -justify left -font $tfnt -tag  "task-$id taskFG taskWorker taskVariable-$id"
	    $ptvCanvas create text [expr $x + 4+$nameWidth ] [expr $y+11] -text [format "%3d%%" $res] \
		-anchor sw -justify left -font $tfnt -tag  "task-$id taskFG taskWorker taskVariable-$id"
	}
	incr r
	if { $r >= $rows } {
	    incr c 
	    set r 0
	}
    }

    if {  $ptvTaskWindSize($id) ==0 } { 
	for { set c 1 } { $c <=$cols } { incr c } {
	    set x [expr $xk0 + 64 + $c *$colWidth ]
	    $ptvCanvas create line $x  [expr $yk0 +  2] $x [expr $yk0 +  2 + $rows *$rowHeight ] -fill black \
		-tag  "task-$id taskFG taskWorker taskVariable-$id"
	}
    }
    
    set yk0 [expr  [lindex [$ptvCanvas bbox taskVariable-$id ] 3 ] +2 ]

    $ptvCanvas create text [expr $xk0 +6] [expr $yk0 +2]  -text "$_projText(ptvDescr):"\
	-anchor nw -justify left -font $fnt10 -tag "task-$id taskFG taskDescr taskDescr-$id taskVariable-$id"
    $ptvCanvas create text [expr $xk0 +66]  [expr $yk0 +2] -text $descr -width [expr $mwidth -4] \
	-anchor nw -justify left -font $fnt10 -tag "task-$id taskFG taskDescr taskDescr-$id taskVariable-$id"
    set yk1 [expr  [lindex [$ptvCanvas bbox taskVariable-$id ] 3 ] +2 ]
    $ptvCanvas create rectangle [expr $xk0+4] $yk0 [expr $xk1-4] $yk1 \
	-fill grey96  -outline black -tag "task-$id taskFG  taskDescrBG-$id taskVariable-$id"
    $ptvCanvas lower taskDescrBG-$id taskDescr-$id

    set mileAnz 0 
    foreach ms $miles {
	set name [ lindex $ms 1]
	set pos  [ lindex $ms 2]
	if { $pos == 0 || $name == "" } { continue }
	incr mileAnz
    }
    if { $mileAnz >0 } {
	global todoStatIcon
	set yk0 [expr $yk1 + 2 ]

	$ptvCanvas create rectangle [expr $xk0+4] $yk0 [expr $xk1-4] [expr $yk0 + 4 +$mileAnz*$rowHeight ] \
	    -fill grey96  -outline black -tag "task-$id taskFG  taskMilesBG-$id taskVariable-$id"
	$ptvCanvas create text [expr $xk0 +6] [expr $yk0 +2]  -text "$_projText(ptvMileStones) :"\
	    -anchor nw -justify left -font $fnt10 -tag "task-$id taskFG taskMiles  taskVariable-$id"
	
	set nr 1
	set l 0
	foreach ms $miles {
	    set stat [ lindex $ms 0]
	    set name [ lindex $ms 1]
	    set pos  [ lindex $ms 2]
	    if { $pos == 0 || $name == "" } { 
		incr nr
		continue 
	    }

	    if { $stat == 1 } {
		$ptvCanvas create image [expr $xk0 +68] [expr $yk0 +2 +$l*13] -anchor nw -image  [image create pixmap -data  $todoStatIcon(DONE-small)] -tag "task-$id taskFG taskMiles taskVariable-$id"
	    } else { 
		$ptvCanvas create image [expr $xk0 +68] [expr $yk0 +2 +$l*13] -anchor nw -image  [image create pixmap -data  $todoStatIcon(WIP-small)] -tag "task-$id taskFG taskMiles taskVariable-$id"
	    }

	    set nam [ strTrunc3 $name $milesWidth $fnt10 [string length $name]]
	    $ptvCanvas create text [expr $xk0 +86]  [expr $yk0 +2+$l*13] -text $nam \
		-anchor nw -justify left -font $fnt10 -tag "task-$id taskFG taskMiles taskVariable-$id"
	    if {  $ptvTaskWindSize($id) ==0 } { 
		$ptvCanvas create text [expr $xk0 +86+ $milesWidth]  [expr $yk0 +2+$l*13] \
		    -text [format "= %3d %%" $pos ] \
		    -anchor nw -justify left -font $fnt10 -tag "task-$id taskFG taskMiles taskVariable-$id"
	    }
	    incr l
	}
    }
    set nbottom  [lindex  [$ptvCanvas bbox taskVariable-$id] 3]
    set ptvTaskExpand($id) [expr $nbottom -$bottom]
   
}

proc ptvDrawTaskStatusBox { xk0 yk0 xk1 id } {
    global ptvCanvas todoStatIcon  mileStoneTranspIcon
    global ptvDepSource fnt10 fnt8 bfnt10
    global tdd_todoList
    global ptvTaskDepSourceX  ptvTaskDepSourceY ptvTaskDepSourceCount

    set compl      [lindex $tdd_todoList($id) 6 ]
    set mileStones [lindex $tdd_todoList($id) 12 ]

#    puts "Status box for $id"
    set list [ $ptvCanvas bbox task-$id]
    # -4 for the missing windowframe
    set sx [expr  [ lindex $list 0] -4 ]
    set sy [ lindex $list 1]


    $ptvCanvas create rectangle [expr $xk0+4] [expr $yk0 +2] [expr $xk1-4] [expr $yk0 + 45]\
	-fill grey96  -outline black -tag "task-$id taskFG  taskStatusBox-$id"

    set mwidth [ expr ($xk1 -6) - ($xk0 +6) ]

    ptvDrawIconBox $ptvCanvas [expr $xk1-24] [expr $yk0+4] 12 12 "task-$id taskStatusBox-$id taskDepDONE taskFG" $todoStatIcon(DONE-small)
    ptvDrawIconBox $ptvCanvas [expr $xk1-44] [expr $yk0+4] 12 12 "task-$id taskStatusBox-$id taskDepKILL taskFG" $todoStatIcon(KILL-small)
    ptvDrawIconBox $ptvCanvas [expr $xk1-64] [expr $yk0+4] 12 12 "task-$id taskStatusBox-$id taskDepHOLD taskFG" $todoStatIcon(HOLD-small)
    ptvDrawIconBox $ptvCanvas [expr $xk1-84] [expr $yk0+4] 12 12 "task-$id taskStatusBox-$id taskDepDELEY taskFG" $todoStatIcon(DELAY-small)
    set x [expr $xk0 + int(floor($mwidth/2))-6]
    if { $x >  [expr $xk1-104] } { set x [expr $xk1-108] }
    ptvDrawIconBox $ptvCanvas $x [expr $yk0+4] 12 12 "task-$id taskStatusBox-$id taskDepWIP taskFG" $todoStatIcon(WIP-small)
    set ptvTaskDepSourceX($id-ONDONE)  [expr $xk1-18 - $sx]
    set ptvTaskDepSourceX($id-ONKILL)  [expr $xk1-38 - $sx]
    set ptvTaskDepSourceX($id-ONHOLD)  [expr $xk1-58 - $sx]
    set ptvTaskDepSourceX($id-ONDELAY) [expr $xk1-78 - $sx]
    set ptvTaskDepSourceX($id-ONWIP)   [expr $x - $sx]
    set ptvTaskDepSourceY($id-ONDONE)  [expr $yk0+23 - $sy]
    set ptvTaskDepSourceY($id-ONKILL)  [expr $yk0+23 - $sy]
    set ptvTaskDepSourceY($id-ONHOLD)  [expr $yk0+23 - $sy]
    set ptvTaskDepSourceY($id-ONDELAY) [expr $yk0+23 - $sy]
    set ptvTaskDepSourceY($id-ONWIP)   [expr $yk0+23 - $sy]
    set ptvTaskDepSourceCount($id) 0

    $ptvCanvas create rectangle [expr $xk0+6] [expr $yk0 +23] [expr $xk1-6] [expr $yk0 + 36]\
	-fill "#704C0A" -outline black -tag "task-$id taskFG taskStatusBox-$id taskComplete" 
    $ptvCanvas create rectangle [expr $xk0+6] [expr $yk0 +23] [expr $xk0+6 + int(floor($mwidth*$compl/100.))] [expr $yk0 + 36]\
	-fill "#FFAE1A" -outline black -tag "task-$id taskFG taskStatusBox-$id taskComplete" 
    set ptvTaskDepSourceX($id-ONCOMPL)  "[expr $xk0 +6 -  $sx] $mwidth"
    set ptvTaskDepSourceY($id-ONCOMPL)   [expr $yk0+38 - $sy]


    
    set ptvTaskDepSourceX($id-ONMILES)  ""
    set ptvTaskDepSourceY($id-ONMILES)   [expr $yk0+46 - $sy]
    set nr 1
    foreach ms $mileStones {
	set stat [ lindex $ms 0]
	set name [ lindex $ms 1]
	set pos  [ lindex $ms 2]
	if { $pos == 0 || $name == "" } { 
	    incr nr
	    lappend  ptvTaskDepSourceX($id-ONMILES)   0 
	    continue 
	}
	
	set xk [expr $xk0+6 + int(floor($mwidth * $pos/100.)) -12 ] 
	lappend  ptvTaskDepSourceX($id-ONMILES)  [expr $xk -$sx -2]

	$ptvCanvas  create image $xk [expr $yk0 + 45 ] -anchor s \
	    -image  [image create pixmap -data  $mileStoneTranspIcon] \
	    -tag "task-$id taskFG taskStatusBox-$id taskMileStone taskMileStone-$id taskMileStone-$id-$nr" 
	if { $stat == 0 } {
	    $ptvCanvas  create text $xk  [expr $yk0 + 44 ] -anchor s -justify center -text $nr -font $bfnt10  \
		-tag "task-$id taskFG taskStatusBox-$id taskMileStone taskMileStone-$id taskMileStone-$id-$nr"  -fill red3 
	} else { 
	    $ptvCanvas  create text $xk  [expr $yk0 + 44 ] -anchor s -justify center -text $nr -font $fnt10  \
		-tag "task-$id taskFG taskStatusBox-$id taskMileStone taskMileStone-$id taskMileStone-$id-$nr"   -fill green4
	}

	#find text border
	set spos 101
	foreach ms2 $mileStones {
	    set pos2  [ lindex $ms2 2]
	    if { $pos2 > $pos  && $pos2 < $spos } { set spos $pos2 }
	}	

	if { $spos < 101 } {
	    set xs [ expr $xk0+6 + int(floor($mwidth * $spos/100.)) -20 ] 
	} else {
	    set xs [ expr $xk0+6 +$mwidth]
	}
	set tw [ expr $xs - ($xk+8) ]
	set nam [ strTrunc3 $name $tw $fnt8 [string length $name]]
	$ptvCanvas  create text [expr $xk + 8]  [expr $yk0 + 46 ] -anchor sw -justify left -text $nam\
	    -font $fnt8 -tag "task-$id taskFG taskStatusBox-$id taskMileStone-$id taskMileStone-$id-$nr" 
	incr nr
    }
    return [expr $yk0 + 47 ]
}

proc ptvDrawTaskEmptyStatusBox {xk0 yk0 xk1 id } {
    global ptvTaskDepSourceX ptvTaskDepSourceY ptvTaskDepSourceCount ptvCanvas

    set list [ $ptvCanvas bbox task-$id]
    puts "EMPTY Status box for $id //$list//"
#    if { $list == "" } { return }
#    puts "drawing EMPTY Status box for $id"
    # -4 for the missing windowframe
    set sx [ lindex $list 0]
    set sx1 [ lindex $list 2]
    set sy [ lindex $list 1]
    set sy1 [ lindex $list 3]

    set xkd [expr ($xk1 - $sx)-4]
    set ykd [expr ($sy1 -$sy) +1]
    set ptvTaskDepSourceX($id-ONDONE)  $xkd
    set ptvTaskDepSourceX($id-ONKILL)  $xkd
    set ptvTaskDepSourceX($id-ONHOLD)  $xkd
    set ptvTaskDepSourceX($id-ONDELAY) $xkd
    set ptvTaskDepSourceX($id-ONWIP)   $xkd
    set ptvTaskDepSourceY($id-ONDONE)  $ykd
    set ptvTaskDepSourceY($id-ONKILL)  $ykd
    set ptvTaskDepSourceY($id-ONHOLD)  $ykd
    set ptvTaskDepSourceY($id-ONDELAY) $ykd
    set ptvTaskDepSourceY($id-ONWIP)   $ykd
    set ptvTaskDepSourceX($id-ONCOMPL)  "$xkd $xkd"
    set ptvTaskDepSourceY($id-ONCOMPL)  $ykd
    set ptvTaskDepSourceX($id-ONMILES)  "$xkd $xkd $xkd $xkd $xkd $xkd $xkd $xkd"
    set ptvTaskDepSourceY($id-ONMILES)  $ykd
    set ptvTaskDepSourceCount($id) 0

}

proc ptvDrawIconBox {cwin  x y dx dy tag icon } {
    ptvDrawRelief $cwin $x $y [ expr $x+$dx+4] [expr $y+$dy+4] $tag grey96
    set xm [ expr $x + int (floor($dx/2)) +2 ]
    $cwin create image $xm [expr $y +2] -anchor n -image  [image create pixmap -data  $icon] -tag $tag
}
proc ptvDrawRelief {cwin  x0 y0 x1 y1 tag bgcolor } {
    
    $cwin create rectangle $x0 $y0 $x1 $y1 -outline grey75 -fill grey86 -tag $tag
    incr x0 
    incr y0 
    incr x1 -1
    incr y1 -1
    $cwin create line $x0 $y1 $x0 $y0 $x1 $y0 -fill white -tag $tag
    $cwin create line $x0 $y1 $x1 $y1 $x1 $y0 -fill black -tag $tag
    incr x0 
    incr y0
    incr x1 -1
    incr y1 -1
    $cwin create rectangle $x0 $y0 $x1 $y1 -outline grey92 -fill $bgcolor -tag $tag
}



proc ptvCoord2TaskID { x y} {
    global ptvCanvas
    set idx [$ptvCanvas find closest [$ptvCanvas canvasx $x] [$ptvCanvas  canvasy $y ] ]
    set tags [$ptvCanvas gettags $idx ]
    set nr [lsearch -glob $tags {task-*} ]
    set id [ lindex [split [lindex $tags $nr] -] 1 ]
    return $id
}
proc ptvCoord2ProjID { x y} {
    global ptvCanvas

    set idx [$ptvCanvas find closest [$ptvCanvas canvasx $x] [$ptvCanvas  canvasy $y ] ]
    set tags [$ptvCanvas gettags $idx ]
    set nr [lsearch -glob $tags {proj-*} ]
    set id "-[ lindex [split [lindex $tags $nr] -] 1 ]"
    return $id
}

proc ptvRecordButtonPressObj { x y arg type} {
    global depSourceType
    global depSourceTask
    global depSourceX depSourceY

    if { $type  == "task" } {
	set id [ptvCoord2TaskID $x $y ]
    } else {
	set id [ptvCoord2ProjID $x $y ]
    }
#    puts "press   $x $y $arg $type -> $id   ($depSourceTask-$depSourceType)"
    if { $arg == "Proj" || $arg == "Task" } {
	if  { $depSourceTask != $id } {
	    set depSourceTask $id 
	    set depSourceType $arg
	    set depSourceX $x
	    set depSourceY $y
	}
    } else {
	set depSourceTask $id 
	set depSourceType $arg
	set depSourceX $x
	set depSourceY $y
    }
#    puts "\t\t\t\t=>$depSourceTask-$depSourceType"
    
}
proc ptvTaskAction { x y  arg  type} {
    global ptvCanvas 
    global depSourceType
    global depSourceTask
    global depDestType
    global depDestTask
    
#    puts "RELEASE [ $ptvCanvas gettags current] depSource=$depSourceTask-$depSourceType"
    if { $depSourceTask == "" } { 
	set depSourceType ""
	set depSourceTask ""
	return 
    }
    set id [ptvCoord2TaskID $x $y ]
    if { $id == "" } {
	set id [ptvCoord2ProjID $x $y ]
    }
#    puts "release $x $y $arg $type -> $id"
    if { $id == ""  || $id == "-" } {
	set depSourceType ""
	set depSourceTask ""
	return
    }
    set depDestType $arg
    set depDestTask $id 

    if { $depDestTask != $depSourceTask  } {
#	puts "created dep : $depDestTask != $depSourceTask || $depDestType != $depSourceType "
	global tdd_todoPID depSourceX depSourceY ptvProjDepSourceX ptvTaskDepSourceX
	set depType "ON$depSourceType"
	set mx [ $ptvCanvas canvasx $depSourceX]
	if { $depSourceType == "MILES" } { 
	    set idxList [$ptvCanvas find withtag taskMileStone-$depSourceTask]
	    set diff 9999999.
	    set val -1
#	    puts "MS: $idxList"
	    foreach idx $idxList { 
		if { [ $ptvCanvas type $idx] == "image" } {
		    set ko  [ $ptvCanvas bbox $idx]
		    set d [expr abs([lindex $ko 0] + [lindex $ko 2] - 2* $mx)]
#		    puts "MS:  idx=$idx d=$d diff=$diff  [$ptvCanvas type $idx]  @ $ko ($depSourceX -> $mx)"
#		    if { $d < $diff } {
			set diff $d
			regexp {taskMileStone-[0-9]*-[0-9]*} [$ptvCanvas gettags $idx]  tag
			set val [lindex [split $tag -] 2]
			incr val -1
#			puts "MS:        -> $val"
		    }
		}
	    }
	} elseif { $depSourceType == "COMPL" } {
	    if { $depSourceTask <0 } {
		set id  [expr -1*$depSourceTask]
		set fx0 [lindex  [$ptvCanvas bbox proj-$id ] 0]
		set dx  [expr ($mx - $fx0) - [lindex $ptvProjDepSourceX($id-$depType) 0] ]
		set val [expr int(floor(double($dx)/double([ lindex $ptvProjDepSourceX($id-$depType) 1])))]
	    } else {
		set fx0 [lindex  [$ptvCanvas bbox task-$depSourceTask ] 0]
		set dx  [expr ($mx - $fx0) - [lindex $ptvTaskDepSourceX($depSourceTask-$depType) 0] ]
		set val [expr int(floor(100.*double($dx)/double([ lindex $ptvTaskDepSourceX($depSourceTask-$depType) 1])))]
#		puts "dx=$dx - > val=$val <- [ lindex $ptvTaskDepSourceX($depSourceTask-$depType) 1]"
	    }
	} else { set val 0 }
	
	if { $val >= 0} { 
	    set addParent $tdd_todoPID($depDestTask)
#	    puts "Creating DEP : $depSourceTask --> $depDestTask with $depType  @ $val   in $addParent"
	    if { [modifyDep add $addParent $depSourceTask $depDestTask $depType $val] >=0 } {
		ptvDrawDep $depDestTask $depSourceTask  $depType $val
	    }
	}
	global tdd_projList
#	puts "PROJ 1: $tdd_projList(1)\n"
#	puts "PROJ 2: $tdd_projList(2)\n"
	
	set depSourceType ""
	set depSourceTask ""
    } else {
#	puts "local Action : '$depSourceType' / '$depDestType' "
# 	if { $depDestType != $depSourceType  } {
# 	    set depSourceType ""
# 	    set depSourceTask ""
	    
# 	} else {
	    switch $depSourceType {
		Expand {
		    if { $type == "task" } {
			ptvExpandOrCollapsTask $id 
		    } else {
			global projExpanded
			set projExpanded([expr -1*$id]) [expr $projExpanded([expr -1*$id]) ^ 1]
			ptvReDraw 0
			updateGrid 0
			#		    ptvExpandOrCollapsProj $id 
		    }
		}
		Modify { 
		    if { $id >= 0 } {
			setCurrentTODO $id
			drawChangeTODO
		    } else {
			setCurrProj [expr -1*$id]
			drawPED
		    }
		} 
		Memo   { 
		    if { $id >= 0 } {
			drawProjManagerMemoPad $id TODO
		    } else {
			drawProjManagerMemoPad [expr -1* $id] PROJ 
		    }
		} 
		Status { puts "change Status" }
		MILES  { puts "toogle milestone" }
		COMPL  { puts "change completion" }
	    }
	    set depSourceType ""
	    set depSourceTask ""
#	}
    }
}

proc commonParent {src dest} {
    global taskParents projParents

    if { $src < 0 } {
	set srcParents $projParents([expr -$src])
    } else {
	set srcParents $taskParents($src)
    }
    if { $dest < 0 } {
	set destParents $projParents([expr -$dest])
    } else {
	set destParents $taskParents($dest)
    }
    if { $srcParents == ""  || $destParents == "" } { return -1 }
    if { [lindex $srcParents 0] != [lindex $destParents 0] } { return -1 }
    set parent [lindex $srcParents 0]
    
    set anz [llength $srcParents]
    set danz [llength $destParents]
    if { $anz > $danz } { set anz $danz }

    for { set p 1 } { $p < $anz } { incr p } {
	if { [lindex $srcParents $p] != [lindex $destParents $p] } {
	    break
	} else {
	    set parent [lindex $srcParents $p]
	}
    }
    return $parent
}

proc addDepToProj {pid src dest type val } {
    global tdd_projList
    
    set tList [lindex $tdd_projList($pid) 12 ]
    set anz [llength $tList]
    for { set p 0 } { $p < $anz } { incr p } {
	set ent [lindex $tList $p]
	set tid [lindex $ent 0]
	if { $tid == $dest } {
	    set depList [lindex $ent 1]
	    set dep "$src $type $val"
	    lappend depList $dep
	    set ent "$tid {$depList} [lindex $ent 2]"
	    set tList [lreplace $tList $p $p $ent]
	    set tdd_projList($pid) [ lreplace $tdd_projList($pid) 12 12 $tList ]
	    #FIXEM send 2 server
	    return 0
	}
    }
    return -1
}
proc modifyDep {action pid src dest type val} {
    global tdd_projList
    
    set tList [lindex $tdd_projList($pid) 12 ]
    set anz [llength $tList]
    # puts "tList=/$tList/"
    for { set p 0 } { $p < $anz } { incr p } {
	set ent [lindex $tList $p]
	set tid [lindex $ent 0]
	# puts "ent=/$ent/->$tid"
	if { $tid == $dest } {
	    set depList [lindex $ent 1]
	    # puts "$tid -> //$ent// -> // $depList//"
	    if { $action== "del" } {
		set done 0
		set depAnz [ llength $depList ]
		for { set dp 0} { $dp < $depAnz} { incr dp } {
		    set dep [lindex $depList $dp]
		    # puts "\tcheck dep /$dep/ vs /$src/$type/$val/"
		    if { $src == [lindex $dep 0] } {
			# puts "\t\t\tid passed"
			if { $type == [lindex $dep 1] } {
			    # puts "\t\t\ttype passsed"
			    if { $val == [lindex $dep 2] } {
				# puts "\t\t\t\tfound dep /$dep/"
				set depList [lreplace $depList $dp $dp ]
				set done 1
				break
			    }
			}
		    }
		}
		if { $done == 0 } { break }
	    } elseif { $action == "add" } {
		set dep "$src $type $val"
		lappend depList $dep
	    }
	    set ent "$tid {$depList} [lindex $ent 2]"
	    set tList [lreplace $tList $p $p $ent]
	    set tdd_projList($pid) [ lreplace $tdd_projList($pid) 12 12 $tList ]
	    # puts "->//$tdd_projList($pid)//"
	    return 0
	}
    }
    return -1
}
proc ptvExpandOrCollapsTask { id } {
    global ptvCanvas ptvDateCoords  ptvTaskCoord ptvStartDate ptvStopDate
    global ptvTaskExpand
    global tdd_todoList
    
    # puts "Expand/Collaps $id"

    set yshift [lindex [ $ptvCanvas bbox task-$id ] 3 ]
#    incr yshift 4

    set start   [lindex $tdd_todoList($id) 4]
    if { $start < $ptvStartDate } { set start $ptvStartDate }
    set stop    [lindex $tdd_todoList($id) 5]
    if { $stop > $ptvStopDate } { set stop $ptvStopDate }
    set xk0 $ptvDateCoords($start)
    set xk1 $ptvDateCoords($stop)
    set yk0 $ptvTaskCoord($id)
    set yks $yk0
    incr yks 1
    
    set idxList [$ptvCanvas find withtag  taskExpand-$id ]
    set idx -1
    foreach i $idxList { 
#	puts "$i : '[ $ptvCanvas type $i]'"
	if { [ $ptvCanvas type $i] == "image" } {
	    set idx $i
	    break
	}
    }
#    puts "canvasID of Icon: '$idxList' -> $idx"
    global taskExpandedIcon16  taskCollapsedIcon16
    if { $ptvTaskExpand($id) == 0 } {
	ptvDrawTaskFull $xk0  $xk1 $id

	ptvMoveObj taskStatusBox-$id  0 $ptvTaskExpand($id) 
	ptvResizeObj taskBG-$id 0 $yks 0  $ptvTaskExpand($id) 
	ptvMoveTasks $yshift $ptvTaskExpand($id) 
	if { $idx >=0 } {$ptvCanvas itemconfigure $idx -image  [image create pixmap -data  $taskExpandedIcon16]}
    } else {
	ptvMoveObj taskStatusBox-$id  0 [expr -1 * $ptvTaskExpand($id) ]
	ptvResizeObj taskBG-$id 0 $yks 0 [expr -1 * $ptvTaskExpand($id) ]
	ptvMoveTasks $yshift  [expr -1 * $ptvTaskExpand($id) ] 

	ptvDrawTaskSmall $xk0  $xk1 $id
	if { $idx >=0 } {$ptvCanvas itemconfigure $idx -image  [image create pixmap -data  $taskCollapsedIcon16]}
    }
    updateGrid 0
}

proc ptvCollapsTask { id  } {
    global tdd_todoList

    set start   [lindex $tdd_todoList($id) 4]
    set stop    [lindex $tdd_todoList($id) 5]
    set xk0 $ptvDateCoords($start)
    set xk1 $ptvDateCoords($stop)
    set yk0 $ptvTaskCoord($id)
    
    #FIXME resize and move
}

proc ptvMoveTasks { yt  dy} {
    global ptvCanvas ptvTaskCoord tdd_projList

    foreach id [array names ptvTaskCoord] {
	set y0 [lindex [ $ptvCanvas bbox "task-$id" ] 1 ]
	if { $y0 > $yt } {
	    $ptvCanvas move "task-$id"  0 $dy 
# 	    if { [ $ptvCanvas find withtag depSrc-task-$id] != "" } {
# 		$ptvCanvas move "depSrc-task-$id"  0 $dy 
# 		$ptvCanvas raise depInTask taskFG
# 	    }
	    incr ptvTaskCoord($id) $dy 
	}
    }

    foreach id [array names tdd_projList ]  {
	set y0 [lindex [ $ptvCanvas bbox "proj-$id" ] 1 ]
	# puts "$id: $y0 vs $yt"
	if { $y0 > $yt } {
	    # puts "\t\t move"
	    $ptvCanvas move "proj-$id"  0 $dy 
	} else {
	    set y1 [lindex [ $ptvCanvas bbox "proj-$id" ] 3 ]
	    # puts "$y1 vs $yt"
	    if { $y1 > $yt } {
		# puts "\t\t resize"
		ptvResizeObj proj-$id 0 $yt 0 $dy 
	    }	
	}
    }
    ptvResizeObj dep 0 $yt 0 $dy
#    ptvResizeObj depInTask 0 $yt 0 $dy
    ptvArrangeObjets
}

proc ptvMoveObj { tagList dx dy } {
    global ptvCanvas 

    foreach tag $tagList {
	$ptvCanvas move $tag $dx $dy
    }
}
proc ptvResizeObj { tagList xt yt dx dy } {
    global ptvCanvas 

    foreach tag $tagList {
	set idxList [ $ptvCanvas find withtag $tag ]
	foreach idx $idxList {
	    set clist [ $ptvCanvas coords $idx]
	    set nlist ""
	    set anz [llength $clist]
	    for { set i 0 } { $i <$anz} { incr i } {
		set x [lindex $clist $i]
		if { $x >= $xt } {
		    lappend nlist [expr $x +$dx ]
		} else {
		    lappend nlist $x
		}
		incr i
		set y [lindex $clist $i]
		if { $y >= $yt } {
		    lappend nlist [expr $y +$dy ]
		} else {
		    lappend nlist $y
		}
	    }
	    # puts "$tag -> $idx //$clist -> $nlist//"
	    $ptvCanvas coords $idx $nlist
	}
    }
}

